#!/usr/bin/perl -w

# all dimensions in mm

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

# All dimensions in mm

my $envelope = 'downstream_shielding';
my $file     = 'downstream_shielding.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;


# Downstream beamline is a 4cm thick pipe of lead, with OD = 350 mm

my $inches   = 25.4;

# Torus numbers:
my $TorusZpos        = 151.855*$inches;                 # center of the torus position
my $SteelFrameLength = 94.*$inches/2.0;                 # 1/2 length
my $bp_zpos          = $TorusZpos + $SteelFrameLength ; # back plate z position
my $face_plate_LE    = 1.0*$inches/2.0;

my $thickness = 40.0;
my $dpipe_OR  = 350.0/2.0;
my $dpipe_IR  = $dpipe_OR - $thickness;
my $dpipe_le  = 1000.0;    # 2 meters total length
my $dpipe_sp  = 60.0;      # 3 cm space between pipe and torus

my $lead_shield_thickness = 25.0;

my $pipe_zpos =  $bp_zpos + $dpipe_le + $dpipe_sp + $lead_shield_thickness;


##############################
# LEAD PIPE - after Torus Ring
##############################

sub make_shield_pipe
{

 $detector{"name"}         = "shield_pipe_after_torus_ring";
 $detector{"mother"}       = "root";
 $detector{"description"}  = "Pipe after Torus Ring";
 $detector{"pos"}          = "0*mm 0.0*mm $pipe_zpos*mm";
 $detector{"rotation"}     = "0*deg 0*deg 0*deg";
 $detector{"color"}        = "993333";
 $detector{"type"}         = "Tube";
 $detector{"dimensions"}   = "$dpipe_IR*mm $dpipe_OR*mm $dpipe_le*mm 0.0*deg 360*deg";
 $detector{"material"}     = "StainlessSteel";
 $detector{"mfield"}       = "no";
 $detector{"ncopy"}        = 1;
 $detector{"pMany"}        = 1;
 $detector{"exist"}        = 1;
 $detector{"visible"}      = 1;
 $detector{"style"}        = 1;
 $detector{"sensitivity"}  = "no";
 $detector{"hit_type"}     = "";
 $detector{"identifiers"}  = "";

 print_det(\%detector, $file);
}


################################
# LEAD SHIELD - after Torus Ring
################################

my $nplanes = 4;
my $ColdHubIR  =  2.38*$inches ;     # total IR, OR as from the drawings in M:\hallb_eng\CLAS12\Magnets\Torus\Torus April 08 parts

# Numbers coming from ROOT macro
# Corner:            1              2            3             4
my @zplane   = (     0.0    ,     $lead_shield_thickness   ,    $lead_shield_thickness + 0.1     ,   300.0     );
my @oradius  = (   170.0    ,     170.0   ,     110.0    ,   110.0     );
my @iradius  = ( $ColdHubIR ,  $ColdHubIR ,   $ColdHubIR ,  $ColdHubIR );
my $zstart   = $bp_zpos + 1.0;

sub make_lead_shield
{

 $detector{"name"}        = "lead_shield";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Lead Shielding - Torus to Lead Pipe";
 $detector{"pos"}         = "0*mm 0.0*mm $zstart*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "0000ff";
 $detector{"type"}      = "Polycone";

 my $dimen = "0.0*deg 360*deg $nplanes";
 for(my $i = 0; $i <$nplanes ; $i++)
 {
    $dimen = $dimen ." $iradius[$i]*mm";
 }
 for(my $i = 0; $i <$nplanes ; $i++)
 {
    $dimen = $dimen ." $oradius[$i]*mm";
 }
 for(my $i = 0; $i <$nplanes ; $i++)
 {
    $dimen = $dimen ." $zplane[$i]*mm";
 }
 $detector{"dimensions"}  = $dimen;
 $detector{"material"}    = "Lead";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}






#########################################
# Vacuum Pipe: inside and after the torus
#########################################
my $TIR      = 37.6;                    # Torus ID is 75.2 mm
my $pthick   = 2.0;                     # Pipe thickness through the Torus Ring
my $gap      = 0.1;                     # gap between Torus ring and vacuum pipe
my $vpipe_le = 3000.0;                  # 6 meters long
my $pipe_back_shift = 39.6;             # to meet the moeller absorber
#my $vpipe_z  = $TorusZpos + $vpipe_le - $SteelFrameLength - $pipe_back_shift;  # z position - to meet the moeller absorber
#my $vpipe_OR = $TIR -  $gap;            #
my $vpipe_z  = $TorusZpos + $vpipe_le - $SteelFrameLength;  # z position - to meet the moeller absorber
my $vpipe_OR = 60.;            #
my $vpipe_MR = 41.;     #
my $vpipe_IR = 30.;     #

sub make_vacuum_pipe
{

 $detector{"name"}         = "downstream_vacuum_pipe";
 $detector{"mother"}       = "root";
 $detector{"description"}  = "Pipe after Torus Ring";
 $detector{"pos"}          = "0*mm 0.0*mm $vpipe_z*mm";
 $detector{"rotation"}     = "0*deg 0*deg 0*deg";
 $detector{"color"}        = "999966";
 $detector{"type"}         = "Tube";
 $detector{"dimensions"}   = "0*mm $vpipe_OR*mm $vpipe_le*mm 0.0*deg 360*deg";
 $detector{"material"}     = "Tungsten";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);

 # Alimunum inside
 $detector{"name"}         = "adownstream_vacuum_pipe";
 $detector{"mother"}       = "downstream_vacuum_pipe";
 $detector{"description"}  = "Aluminum Pipe after Torus Ring";
 $detector{"pos"}          = "0*mm 0.0*mm 0*mm";
 $detector{"rotation"}     = "0*deg 0*deg 0*deg";
 $detector{"color"}        = "87AFC7";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}   = "0*mm $vpipe_MR*mm $vpipe_le*mm 0.0*deg 360*deg";
 $detector{"material"}     = "Aluminum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);

 
 # Vacuum inside
 $detector{"name"}         = "vdownstream_vacuum_pipe";
 $detector{"mother"}       = "adownstream_vacuum_pipe";
 $detector{"description"}  = "Vacuum Pipe after Torus Ring";
 $detector{"pos"}          = "0*mm 0.0*mm 0*mm";
 $detector{"rotation"}     = "0*deg 0*deg 0*deg";
 $detector{"color"}        = "aaffff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}   = "0*mm $vpipe_IR*mm $vpipe_le*mm 0.0*deg 360*deg";
 $detector{"material"}     = "Vacuum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

print_det(\%detector, $file);
}



make_shield_pipe();
make_lead_shield();
make_vacuum_pipe();



