#!/usr/bin/perl

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;


my $envelope = 'FT';
my $file     = 'FT.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;



use Getopt::Long;
use Math::Trig;

my $inches = 25.4;
my $SteelFrameLength = 94.29*$inches/2.0;   # 1/2 length
my $col              = 'ffff9b';
my $TorusZpos= 151.855*$inches;                 # center of the torus position
my $ipipe_ir = 2.42*$inches;
my $ipipe_or = 2.50*$inches;
my $ipipe_le = 93.5*$inches/2.0;
sub make_ipipe
{
 $detector{"name"}        = "torus_inner_pipe";
 $detector{"mother"}      = "root";
 $detector{"description"} = "FT Beam Line";
 $detector{"pos"}         = "0.0*cm 0.0*cm $TorusZpos*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ff8883";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$ipipe_ir*mm $ipipe_or*mm $ipipe_le*mm 0*deg 360*deg";
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}


# Front and back Plates
my $face_plate_IR = 1.56*$inches + .1;  # Allow microgap to avoid overlaps
my $face_plate_OR = 7.64*$inches - .1;
my $face_plate_LE = 1.18*$inches/2.0;
my $fp_zpos       = $TorusZpos - $SteelFrameLength - 0.1; # Allow microgap to avoid overlaps
my $bp_zpos       = $TorusZpos + $SteelFrameLength + 0.1;

sub make_face_plate
{
 $detector{"name"}        = "Torus_front_plate";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Torus Front Plate";
 $detector{"pos"}         = "0.0*cm 0.0*cm $fp_zpos*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = $col;
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$face_plate_IR*mm $face_plate_OR*mm $face_plate_LE*mm  0.0*deg 360.0*deg";
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}

sub make_face_plate2
{
 $detector{"name"}        = "Torus_back_plate";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Torus Back Plate";
 $detector{"pos"}         = "0.0*cm 0.0*cm $bp_zpos*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = $col;
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$face_plate_IR*mm $face_plate_OR*mm $face_plate_LE*mm  0.0*deg 360.0*deg";
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}




###########################################################################################
# Define FT Mother Volume
my $torus_z    = 2663.; # position of the front face of the Torus ring (set the limit in z)
my $nplanes_FT = 4;
#my @z_plane_FT = ( 1777., 2150., 2150., $torus_z);
my @z_plane_FT = ( 1796., 2150., 2150., $torus_z);
my @oradius_FT = (  270.,  270.,  149.,     149.);
my @iradius_FT = (    0.,    0.,    0.,       0.);

sub make_FT
{
 $detector{"name"}        = $envelope;
 $detector{"mother"}      = "root";
 $detector{"description"} = "Forward Tagger";
 $detector{"pos"}         = "0.*mm 0.*mm 0.*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "1437f4";
 $detector{"type"}        = "Polycone";

 my $dimen = "0.0*deg 360*deg $nplanes_FT";
 for(my $i = 0; $i <$nplanes_FT ; $i++)
{
    $dimen = $dimen ." 0.0*mm";
}
 for(my $i = 0; $i <$nplanes_FT ; $i++)
{
    $dimen = $dimen ." $oradius_FT[$i]*mm";
}
 for(my $i = 0; $i <$nplanes_FT ; $i++)
{
    $dimen = $dimen ." $z_plane_FT[$i]*mm";
}
 $detector{"dimensions"} = $dimen;
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}


###########################################################################################
# Define the tube between Calorimeter and Torus Inner Ring

#my $Tube_IR         =  56.0;
#my $Tube_OR         =  89.0;
my $Tube_IR         =  30.0;
my $Tube_OR         =  55.0;
my $Tube_LT         = 365.6;
my $back_flange_OR  = 126.5;
my $front_flange_OR = 148.0;
my $flange_TN       =  15.0;

# define positions based on z of torus inner ring front face
my $Tube_end  = $torus_z-0.5;    # leave 0.5 mm to avoid overlaps
my $Tube_z1   = $Tube_end - $flange_TN;
my $Tube_z2   = $Tube_z1 - $Tube_LT;
my $Tube_beg  = $Tube_z2 - $flange_TN;

my $nplanes_Tube = 6;
my @z_plane_Tube = (        $Tube_beg,         $Tube_z2, $Tube_z2, $Tube_z1,        $Tube_z1,       $Tube_end);
my @oradius_Tube = ( $front_flange_OR, $front_flange_OR, $Tube_OR, $Tube_OR, $back_flange_OR, $back_flange_OR);
my @iradius_Tube = (         $Tube_IR,         $Tube_IR, $Tube_IR, $Tube_IR,        $Tube_IR,        $Tube_IR);
sub make_FT_2_Torus_Tube
{
 $detector{"name"}        = "FT_2_Torus_Tube";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "Tube from FT to Torus";
 $detector{"pos"}         = "0.0*cm 0.0*cm 0.0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "99cc00";
 $detector{"type"}        = "Polycone";

 my $dimen = "0.0*deg 360*deg $nplanes_Tube";
 for(my $i = 0; $i <$nplanes_Tube ; $i++)
{
    $dimen = $dimen ." $iradius_Tube[$i]*mm";
}
 for(my $i = 0; $i <$nplanes_Tube ; $i++)
{
    $dimen = $dimen ." $oradius_Tube[$i]*mm";
}
 for(my $i = 0; $i <$nplanes_Tube ; $i++)
{
    $dimen = $dimen ." $z_plane_Tube[$i]*mm";
}
 $detector{"dimensions"}  = $dimen;
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}


###########################################################################################
# Define the Flange on the back of the FT

my $Flange_IR  =  30.0;
my $Flange_MR  =  57.0;
my $Flange_OR  =  89.0;
my $Flange_DR  = 220.0;
my $Flange_LT  = 168.5;
my $Flange_T1  =  20.0;
my $Flange_T2  =  35.0;

my $Flange_end  = $Tube_beg -0.5;    # leave 0.5 mm to avoid overlaps
my $Flange_z1   = $Flange_end - $Flange_T1;
my $Flange_z2   = $Flange_z1  - ($Flange_LT - $Flange_T1 - $Flange_T2);
my $Flange_beg  = $Flange_end - $Flange_LT;

my $nplanes_Flange = 6;
my @z_plane_Flange = ( $Flange_beg, $Flange_z2, $Flange_z2, $Flange_z1, $Flange_z1, $Flange_end);
my @oradius_Flange = (  $Flange_DR, $Flange_DR, $Flange_MR, $Flange_MR, $Flange_OR,  $Flange_OR);
my @iradius_Flange = (  $Flange_IR, $Flange_IR, $Flange_IR, $Flange_IR, $Flange_IR,  $Flange_IR);

sub make_FT_Flange
{
 $detector{"name"}        = "FT_Flange";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "Flange on the back of the FT";
 $detector{"pos"}         = "0.0*cm 0.0*cm 0.0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ccff00";
 $detector{"type"}        = "Polycone";

 my $dimen = "0.0*deg 360*deg $nplanes_Flange";
 for(my $i = 0; $i <$nplanes_Flange ; $i++)
{
    $dimen = $dimen ." $iradius_Flange[$i]*mm";
}
 for(my $i = 0; $i <$nplanes_Flange ; $i++)
{
    $dimen = $dimen ." $oradius_Flange[$i]*mm";
}
 for(my $i = 0; $i <$nplanes_Flange ; $i++)
{
    $dimen = $dimen ." $z_plane_Flange[$i]*mm";
}
 $detector{"dimensions"}  = $dimen;
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}



###########################################################################################
# Define the Calorimeter Tube

my $BLine_LT = 301.5/2;
my $BLine_OR = $Flange_IR + 10.;
my $BLine_IR = $Flange_IR;
my $BLine_Z  = $Flange_beg - $BLine_LT -0.1;

sub make_FT_BLine
{
 $detector{"name"}        = "FT_BLine";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "FT Beam Line";
 $detector{"pos"}         = "0.0*cm 0.0*cm $BLine_Z*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ccff00";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$BLine_IR*mm $BLine_OR*mm $BLine_LT*mm 0.*deg 360.*deg";
 $detector{"material"}    = "G4_W";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}


###########################################################################################
# Define the Calorimeter Back Plate

my $Plate_TN =  35./2;
my $Plate_OR = 205.;
my $Plate_IR = $BLine_OR;
my $Plate_Z  = $Flange_beg - $Plate_TN -0.5;

sub make_FT_Back_Plate
{
 $detector{"name"}        = "FT_Back_Plate";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "FT Back_Plate";
 $detector{"pos"}         = "0.0*cm 0.0*cm $Plate_Z*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "7F9A65";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$Plate_IR*mm $Plate_OR*mm $Plate_TN*mm 0.*deg 360.*deg";
 $detector{"material"}    = "StainlessSteel";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}


###########################################################################################
# Define the coordinates of the focal point of the IC in mm

my $xFocal = 0;
my $yFocal = 0;
my $zFocal = 850;       # put the inner radius at (0.,0.,0)
my $Cback = $Plate_Z - $Plate_TN - 0.5;

# Define the dimensions of the crystals
my $Cwidth  = 15.0  ;  # Length of the sides on front face in mm 
my $Clength = 200.0 ;  # Depth of the crystals in mm

# Define the dimensions of the crystals
my $ltFront = 13.33 ;  # Length of the sides on front face in mm 
my $ltBack  = 16.0  ;  # Length of the sides on the back side in mm
my $depth   = 160.0 ;  # Depth of the crystals in mm

# Determine the focal lenght and the aperature in both directions for the cystals
my $dAlpha = 2.0 * atan( ( $ltBack - $ltFront ) / ( 2.0 * $depth ) );
my $lFoc   =  ( $ltFront / 2.0 )  / tan( $dAlpha / 2.0 );

# Number of crystals in horizonthal and vertical directions
my $Nx = 23;
my $Ny = 23;


###########################################################################################3




if( ( $Nx % 2 ) == 0 || ( $Nx % 2 ) == 0 )
{
 print STDERR  "I only want to work with odd numbers ... \nExiting \n";
 die -1;
}





my %Slot ;
my %Chan ;

&read_trans_table( \%Slot, \%Chan ) ;


# Loop over all crystals and define their positions
sub make_crystals
{
 for ( my $iX = 1; $iX <= $Nx; $iX++ )
 {
    if( exists $Slot{$iX} )
    {
       for ( my $iY = 1; $iY <= $Ny; $iY++ )
       {
          if( exists $Slot{$iX}{$iY} )
          {
             my %crystPars = &getCrystGeom( $iX, $iY );
             if( scalar %crystPars )
             {

                $detector{"name"}        = $crystPars{"name"};
                $detector{"mother"}      = $crystPars{"mother"};
                $detector{"description"} = $crystPars{"desc"} ;
                $detector{"pos"}         = "$crystPars{'locX'}*mm $crystPars{'locY'}*mm $crystPars{'locZ'}*mm";
                $detector{"rotation"}    = "$crystPars{'phiX'}*deg $crystPars{'phiY'}*deg $crystPars{'phiZ'}*deg";
                $detector{"color"}       = $crystPars{"color"};
                $detector{"type"}        = "$crystPars{'shape'}";
                $detector{"dimensions"}  = "$crystPars{'dx'}*mm $crystPars{'dy'}*mm $crystPars{'dz'}*mm";
                $detector{"material"}    = $crystPars{"mate"};
                $detector{"mfield"}      = $crystPars{"magf"};
                $detector{"ncopy"}       = $crystPars{"ncpy"};
                $detector{"pMany"}       = $crystPars{"pMany"};
                $detector{"exist"}       = $crystPars{"exist"};
                $detector{"visible"}     = $crystPars{"visib"};
                $detector{"style"}       = $crystPars{"style"};
                $detector{"sensitivity"} = $crystPars{"sens"};
                $detector{"hit_type"}    = $crystPars{"hit"};
                $detector{"identifiers"} = $crystPars{"identifier"};

                print_det(\%detector, $file);

             }
          }
       }
    }
 }
}



sub getCrystGeom
{
 my ( $iX, $iY ) = @_;
 my %geomPars = ();


 # Determine the directional angle wrt the three axes
 my $centX = ( int $Nx/2 ) + 1;
 my $centY = ( int $Ny/2 ) + 1;

 
    # Radius at the center of trapezoid from the focal point
    my $inH = cnumber($iX);
    my $inV = cnumber($iY);

#    $geomPars{"name"}        = "IC_CR_" . $inH . "_" . $inV ;
    $geomPars{"name"}        = "FT_CR_" . $iX . "_" . $iY ;
    $geomPars{"desc"}        = "FT Crystal  (h:" . $iX . ", v:" . $iY . ")" ;
    $geomPars{"mother"}      = $envelope ;
    $geomPars{"color"}       = "836FFF" ;
    $geomPars{"shape"}       = "Box" ;
    $geomPars{"mate"}        = "LeadTungsten" ;
    $geomPars{"magf"}        = "no" ;
    $geomPars{"magf"}        = "no" ;
    $geomPars{"ncpy"}        = "1" ;
    $geomPars{"pMany"}       = "1" ;
    $geomPars{"exist"}       = "1" ;
    $geomPars{"visib"}       = "1" ;
    $geomPars{"style"}       = "1" ;
    $geomPars{"sens"}        = "IC" ;	# connected to the sensitive volume described in the proper bank
#   $geomPars{"hit"}         = "IC" ;	# connencted to the hit characteristics defined in clas12_hits_def.txt
    $geomPars{"hit"}         = "FT" ;	# connencted to the hit characteristics defined in clas12_hits_def.txt
    $geomPars{"identifier"}  = "ih manual $iX iv manual $iY" ;

    # these are logical volume parameters for crystals, they do not change
    $geomPars{"dx"}     = sprintf("%.3f", $Cwidth / 2.0)  ;
    $geomPars{"dy"}     = sprintf("%.3f", $Cwidth / 2.0)  ;
    $geomPars{"dz"}     = sprintf("%.3f", $Clength/ 2.0)  ;

    # These are rotaional angles around the 3 different axis
    $geomPars{"phiX"}    = sprintf("%.1f", 0.);
    $geomPars{"phiY"}    = sprintf("%.1f", 0.);
    $geomPars{"phiZ"}    = sprintf("%.1f", 0.);

    $geomPars{"locX"}    = sprintf("%.2f", ($iX - $centX)*$Cwidth);
    $geomPars{"locY"}    = sprintf("%.2f", ($iY - $centY)*$Cwidth);
    $geomPars{"locZ"}    = sprintf("%.2f", $Cback - $Clength/ 2.);

 return  %geomPars ;
}


###########################################################################################
# Define the Calorimeter Front Plate

my $FPlate_TN =  20./2;
my $FPlate_OR = 195.;
my $FPlate_IR = $BLine_OR;
my $FPlate_Z  = $Plate_Z - $Plate_TN - $FPlate_TN - $Clength -1.5;

sub make_FT_Front_Plate
{
 $detector{"name"}        = "FT_Front_Plate";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "FT Front_Plate";
 $detector{"pos"}         = "0.0*cm 0.0*cm $FPlate_Z*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "CCCCFF";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$FPlate_IR*mm $FPlate_OR*mm $FPlate_TN*mm 0.*deg 360.*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";
 print_det(\%detector, $file);
}



#---------------------------------------------------------------------

sub read_trans_table 
{
 my  ($Slot_ptr, $Chan_ptr) = @_;
 my $tr_filename = "trans_table_ft.txt" ;

 # This is Stepan's mapping of the scaler by geom locations
 # in classc2 crate.
 my %sc_addr = ( 5  =>  9,
                 6  => 10,
                 7  =>  1,
                 8  =>  4,
                 9  => 11,
                 10 =>  2,
                 11 =>  6,
                 12 =>  7,
                 13 =>  8,
                 14 =>  3,
                 15 =>  5,
                 16 => 12,
                 17 => 13,
                 18 =>  0
               );

 open ( TRTB, "< $tr_filename" ) or
 die "Sorry, can't open file $tr_filename \n";

 my $line = <TRTB>;

 while ( $line = <TRTB> ) 
 {
    chop $line;
    my @lin_list = split(/\s+/, $line) ;

    my $xpos = $lin_list[1];
    my $ypos = $lin_list[2];
    my $cble = $lin_list[3];
    my $pin  = $lin_list[4];

    my $geom_loc = int( $cble/2 ) + 5;
    my $slot = $sc_addr{$geom_loc};
    my $flag = int( $cble%2 + 0.49 );

    $$Slot_ptr{$xpos}{$ypos} = $slot;
    $$Chan_ptr{$xpos}{$ypos} = $pin+$flag*16;
 }
 close( TRTB );
}

#make_ipipe();
#make_face_plate();
#make_face_plate2();

make_FT();

make_FT_2_Torus_Tube();
make_FT_Flange();
make_FT_BLine();
make_FT_Back_Plate();
make_FT_Front_Plate();

make_crystals();











