#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);

my $envelope = 'MotherVolume';
my $file     = 'MotherVolume.txt';

print "FILE: ",  $file."\n";

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

use Getopt::Long;
use Math::Trig;

$detector{"mfield"}      = "no";
$detector{"ncopy"}       = 1;
$detector{"pMany"}       = 1;
$detector{"exist"}       = 1;
$detector{"visible"}     = 1; # 0 or 1
$detector{"style"}       = 1;
$detector{"sensitivity"} = "no";
$detector{"hit_type"}    = "";
$detector{"identifiers"} = "";

########## Gas Volume ###################

my $numZplanes = 3;
my @zPlane = (  380.0 , 750 , 1750);
my @rInner = (  0 , 0 , 0 );
my @rOuter = (  2500 , 2500 , 2500 );

#my $numZplanes = 3;
#my @zPlane = ( -750 , 750 , 1750);
#my @rInner = ( 0 , 0 , 0 );
#my @rOuter = ( 1850 , 2200 , 2200 );

my $dimensions = "0.0*deg 360.0*deg $numZplanes";

for(my $i = 0; $i <$numZplanes ; $i++)
{
    $dimensions = $dimensions ." $rInner[$i]*mm";
}
for(my $i = 0; $i <$numZplanes ; $i++)
{
    $dimensions = $dimensions ." $rOuter[$i]*mm";
}
for(my $i = 0; $i <$numZplanes ; $i++)
{
    $dimensions = $dimensions ." $zPlane[$i]*mm";
}

sub make_BigGasVolume
{
 $detector{"name"}        = "AHTCC1";
 $detector{"mother"}      = "root";
 $detector{"description"} = "volume containing cherenkov gas";
 $detector{"pos"}         = "0*mm 0*mm 0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ee99ff"; # add a 4 or 5 at the end to make transparent (change visibility above too)
 $detector{"type"}        = "Polycone";
 $detector{"dimensions"}  = "$dimensions";
 #$detector{"material"}    = "CCGas";
 $detector{"material"}    = "Component";

 print_det(\%detector, $file);
}

make_BigGasVolume();

############# Second Volume ######################

my $numZplanes2 = 2;
my @zPlane2 = (  -750 , 400 );
my @rInner2 = (  1400 , 1400 );
my @rOuter2 = (  2500 , 2500 );

my $dimensions2 = "0.0*deg 360.0*deg $numZplanes2";

for(my $i = 0; $i <$numZplanes2 ; $i++)
{
    $dimensions2 = $dimensions2 ." $rInner2[$i]*mm";
}
for(my $i = 0; $i <$numZplanes2 ; $i++)
{
    $dimensions2 = $dimensions2 ." $rOuter2[$i]*mm";
}
for(my $i = 0; $i <$numZplanes2 ; $i++)
{
    $dimensions2 = $dimensions2 ." $zPlane2[$i]*mm";
}

sub make_BigGasVolume2
{
 $detector{"name"}        = "AHTCC2";
 $detector{"mother"}      = "root";
 $detector{"description"} = "volume containing cherenkov gas";
 $detector{"pos"}         = "0*mm 0*mm 0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ff44ff"; # add a 4 or 5 at the end to make transparent (change visibility above too)
 $detector{"type"}        = "Polycone";
 $detector{"dimensions"}  = "$dimensions2";
 #$detector{"material"}    = "CCGas";
 $detector{"material"}    = "Component";

 print_det(\%detector, $file);
}

make_BigGasVolume2();

############ Addition ##################

sub make_GasVolumeFinal
{
 $detector{"name"}        = "HTCC";
 $detector{"mother"}      = "root";
 $detector{"description"} = "gas volume for htcc";
 $detector{"pos"}         = "0*mm 0*mm 0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ff44ff5"; # add a 4 or 5 at the end to make transparent (change visibility above too)
 $detector{"type"}        = "Operation: AHTCC1 + AHTCC2";
 $detector{"dimensions"}  = "0";
 $detector{"material"}    = "CCGas";
 #$detector{"material"}    = "Component";

 print_det(\%detector, $file);
}

make_GasVolumeFinal();
