#!/usr/bin/perl -w
#
# Perl sctipt used to generate a file (EC.txt) that is an input to 
# the shell script go_table which. puts the EC geometry in the mysql 
# database for gemc to use. Modified from original by C.Musalo 7/16/10
# load libraries 

#use strict;
use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);
use Getopt::Long;
use Math::Trig;

# local quantities.

my $envelope = 'LTCC_Mirrors';
my $file     = 'LTCC_Mirrors.txt'; 
my $rmin = 1;
my $rmax = 1000000;


my $inches = 25.4;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;


# All dimensions in cm

my @focal1x = (1..18);
my @focal1y = (1..18);
my @focal2x = (1..18);
my @focal2y = (1..18);
my @centerx = (1..18);
my @centery = (1..18);

my @axisa = (1..18);
my @axisb = (1..18);

my @alpha = (1..18);


my @x12       = (1..18);
my @y12       = (1..18);
my @thetaspan = (1..18);

my @thickness = (1..18);
my @segtheta  = (1..18);



# Read the focal points from file
sub read_foci
{
	my $fname = "foci.txt" ;

	open ( FILE,  $fname ) or
	die "Sorry, can't open file $fname \n";

	my $i    = 0;
	my @lines = <FILE>;
	foreach my $line (@lines)
	{
		chomp($line);
		my @lin_list = split(/[ ]+/,$line);

    $focal1x[$i] = $lin_list[1];
    $focal1y[$i] = $lin_list[2];
    $focal2x[$i] = $lin_list[3];
    $focal2y[$i] = $lin_list[4];
		
		$centerx[$i] = ($focal1x[$i] + $focal2x[$i])/2.0;
		$centery[$i] = ($focal1y[$i] + $focal2y[$i])/2.0;
		
		$alpha[$i]   = atan(($focal2y[$i]-$focal1y[$i])/($focal2x[$i]-$focal1x[$i]))*180/$pi;
		
		#print $i, "  x1f: " , $alpha[$i], "\n";
		$i += 1;
	}	
}

# Read the axis from file
sub read_axis
{
	my $fname = "axis.txt" ;

	open ( FILE,  $fname ) or
	die "Sorry, can't open file $fname \n";

	my $i    = 0;
	my @lines = <FILE>;
	foreach my $line (@lines)
	{
		chomp($line);
		my @lin_list = split(/[ ]+/,$line);

    $axisa[$i] = $lin_list[1];
    $axisb[$i] = $lin_list[2];
		
		my $focal1 = sqrt($axisa[$i]*$axisa[$i] - $axisb[$i]*$axisb[$i]);
		my $focal2 = sqrt( ($focal2y[$i] - $focal1y[$i])*($focal2y[$i] - $focal1y[$i]) + ($focal2x[$i] - $focal1x[$i])*($focal2x[$i] - $focal1x[$i]) )/2 ;
		
		my $check = $focal1 - $focal2;
		
		if($check > 0.0001) 
		{
			#print " WARNING: segment", $i, "   shows a difference in focal points calculation  " ,  $check , "\n";
		}
		$i += 1;
	}	
}


# Read the coordinate of far edge from file
sub read_x12y12
{
	my $fname = "x12y12.dat" ;

	open ( FILE,  $fname ) or
	die "Sorry, can't open file $fname \n";

	my $i    = 0;
	my @lines = <FILE>;
	foreach my $line (@lines)
	{
		chomp($line);
		my @lin_list = split(/[ ]+/,$line);

    $x12[$i] = $lin_list[1];
    $y12[$i] = $lin_list[2];
		
		$thetaspan[$i] = 90.0 - (atan($y12[$i]/$x12[$i]))*180.0/$pi;
		
		#print $thetaspan[$i], "\n";
		
		$i += 1;
	}	
}



sub read_thickness
{
	my $fname = "thicknesses.txt" ;
	open ( FILE,  $fname ) or
	die "Sorry, can't open file $fname \n";

	my $i    = 0;
	my @lines = <FILE>;
	foreach my $line (@lines)
	{
		chomp($line);
		my @lin_list = split(/[ ]+/,$line);

    $thickness[$i] = $lin_list[1];
				
		#print $thickness[$i], "\n";
		
		$i += 1;
	}	
}



sub read_theta
{
	my $fname = "thetas.txt" ;
	open ( FILE,  $fname ) or
	die "Sorry, can't open file $fname \n";

	my $i    = 0;
	my @lines = <FILE>;
	foreach my $line (@lines)
	{
		chomp($line);
		my @lin_list = split(/[ ]+/,$line);

    $segtheta[$i] = $lin_list[3];
				
		#print $segtheta[$i], "\n";
		
		$i += 1;
	}	
}


read_foci();
read_axis();
read_x12y12();
read_thickness();
read_theta();

my $mirrors_thickness = 0.1;
my $nmirrors = 18;

sub build_mirrors_frames
{

	for(my $n=1; $n<=$nmirrors; $n++)
	{
		my $mirrors_length    = $thickness[$n-1]/2.0;

		# outer shell (full)
		$detector{"name"}        = "ael_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Outside $n";
		$detector{"pos"}         = "0*mm 0*mm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg"; 
		$detector{"color"}       = "110088";
		$detector{"type"}        = "EllipticalTube";
		$detector{"dimensions"}  = "$axisa[$n-1]*cm $axisb[$n-1]*cm $mirrors_length*cm";
		$detector{"material"}    = "Air";
		$detector{"material"}    = "Component";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		print_det(\%detector, $file);

		# inner shell (full)
		my $maa = $axisa[$n-1] - $mirrors_thickness;
		my $mab = $axisb[$n-1] - $mirrors_thickness;
		my $ml  = $mirrors_length + 1.0;
		$detector{"name"}        = "bel_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Inside $n";
		$detector{"pos"}         = "0*mm 0*mm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg"; 
		$detector{"color"}       = "110088";
		$detector{"type"}        = "EllipticalTube";
		$detector{"dimensions"}  = "$maa*cm $mab*cm $ml*cm";
		$detector{"material"}    = "Air";
		$detector{"material"}    = "Component";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		print_det(\%detector, $file);

		# Outer - Inner
		$detector{"name"}        = "cel_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Frame $n";
		$detector{"pos"}         = "0*mm 0*mm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg"; 
		$detector{"color"}       = "110088";
		$detector{"type"}        = "Operation: ael_$n - bel_$n";
		$detector{"dimensions"}  = "0";
		$detector{"material"}    = "Air";
		$detector{"material"}    = "Component";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		print_det(\%detector, $file);


		# CHECK: Outer - Inner 
		$detector{"name"}        = "checkel_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Frame $n";
		$detector{"pos"}         = "$centerx[$n-1]*cm $centery[$n-1]*cm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg $alpha[$n-1]*deg"; 
		$detector{"color"}       = "ffbb99";
		$detector{"type"}        = "Operation: ael_$n - bel_$n";
		$detector{"dimensions"}  = "0";
		$detector{"material"}    = "Air";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		#print_det(\%detector, $file);



		# tube with MAX theta angle to subtract
		my $tubespan   = 360.0 - $thetaspan[$n-1];
		my $starttheta =  90.0 + $thetaspan[$n-1];
		my $inv_rot    =  -$alpha[$n-1];
		my $inv_x      = 0 - $centerx[$n-1];
		my $inv_y      = 0 - $centery[$n-1];
		$detector{"name"}        = "del_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Tube with Max Theta $n";
		$detector{"pos"}         = "0*cm 0*cm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg"; 
		$detector{"pos"}         = "$inv_x*cm $inv_x*cm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg $inv_rot*deg"; 
		$detector{"color"}       = "110088";
		$detector{"type"}        = "Tube";
		$detector{"dimensions"}  = "0*m 5*m 0.6*m $starttheta*deg $tubespan*deg";
		$detector{"material"}    = "Air";
		$detector{"material"}    = "Component";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		print_det(\%detector, $file);
	
		# CHECK: tube with MAX theta angle to subtract
		$detector{"name"}        = "checktube_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "Light Threshold Cerenkov Counter Mirror Tube with Max Theta $n";
		$detector{"pos"}         = "0*cm 0*cm 0*mm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg"; 
		$detector{"color"}       = "110088";
		$detector{"type"}        = "Tube";
		$detector{"dimensions"}  = "0*m 5*m 0.6*m $starttheta*deg $tubespan*deg";
		$detector{"material"}    = "Air";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
		#print_det(\%detector, $file);
	
	
	
		my $XPOS = $centerx[$n-1] ;
		my $YPOS = $centery[$n-1] ;
		my $xrot = -90 + $segtheta[$n-1];
		# Subtract tube with MAX theta angle
		$detector{"name"}        = "eel_$n";;
		$detector{"mother"}      = "root";
		$detector{"description"} = "LTCC Mirror $n";
		$detector{"pos"}         = "$XPOS*cm $YPOS*cm 0*mm";
		$detector{"rotation"}    = "ordered: yxz 0*deg $xrot*deg  $alpha[$n-1]*deg"; 
		$detector{"color"}       = "ffff88";
		$detector{"type"}        = "Operation:~ cel_$n - del_$n ";
		$detector{"dimensions"}  = "0*m";
		$detector{"material"}    = "Air_Opt";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "FLUX";
		$detector{"hit_type"}    = "FLUX";
		$detector{"identifiers"} = "Mirror WithSurface: 0    With Finish: 0   Refraction Index: 1108000  With Reflectivity  550000   With Efficiency 400000   WithBorderVolume: MirrorSkin";
		print_det(\%detector, $file);
	
	
	}
	

	
}

build_mirrors_frames();




