#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

use geo qw($pi);

# All dimensions in mm

my $envelope = 'BMT';
my $file     = 'BMT.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();          # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

# General description
my @radius             = (110.000, 120.000, 170.000, 180.000, 230.000, 240.000 );  # center radii of barrel MM
my @nsegments          = (3      , 3      , 3      , 3      , 3      , 3);  # Number of segments in each layer
my @starting_point     = (-84.025, -84.025, -126.038, -126.038, -168.050, -168.050);  # Z starting_point of the moduli
my @Dz_halflength     = (127.800,  127.800, 191.600,  191.600,  254.500,  254.500);  # half length of the moduli along z (35 to 125 acceptance)
#my @Dz_halflength     = (254.500,  254.500, 254.500,  254.500,  254.500,  254.500);  # half length of the moduli along z (35 to 125 acceptance)
my @starting_theta     = (-90    , -90    , -90    , -90    , -90    , -90);  # Starting angle of the first segment
my @dtheta             = (119.0  , 119.0  , 119.0  , 119.0  , 119.0  , 119.0);  # Delta theta, to be defined and refined
my $Dtheta = 120.0;

#######################
# Dimensions (in mm):
# Each segment is a Tube
#######################

my $Epoxy_Width       = 2.000/2.;   # Epoxy width along radius
my $Epoxy_Dz          = 0.000;   # Epoxy z distance to detectors edge (along z)

my $PCB_Width         = 0.200/2.;   # PCB width along radius
my $PCB_Dz            = 0.000;   # PCB z distance to detectors edge (along z)

my $Strips_Width      = 0.015/2.;   # Strips width along radius
my $Strips_Dz         = 3.000;   # Strips z distance to detectors edge (along z)

my $Gas1_Width        = 0.100/2.;   # Gas1 width along radius
my $Gas1_Dz           = 3.000;   # Gas1 z distance to detectors edge (along z)

my $Mesh_Width        = 0.030/2.;   # Mesh width along radius
my $Mesh_Dz           = 3.000;   # Mesh z distance to detectors edge (along z)

my $Gas2_Width        = 2.000/2.;   # Gas2 width along radius
my $Gas2_Dz           = 3.000;   # Gas2 z distance to detectors edge (along z)

my $Drift_Width       = 0.100/2.;   # Drift width along radius
my $Drift_Dz          = 3.000;   # Drift z distance to detectors edge (along z)


# G4 materials
my $epoxy_material     = 'Epoxy';
my $pcboard_material   = 'Epoxy';
my $strips_material    = 'MMStrips';
my $gas_material       = 'MMGas';
my $mesh_material      = 'MMMesh';
my $drift_material     = 'MMMylar';


# G4 colors
my $epoxy_color        = 'e200e1';
my $pcboard_color      = '0000ff';
my $strips_color       = '353540';
my $gas_color          = 'e10000';
my $mesh_color         = '252020';
my $drift_color        = 'fff600';


$pi = 3.141592653589793238;
sub tan { sin($_[0]) / cos($_[0])  }
sub rad { $_[0]*$pi/180.0  }
sub atan {atan2($_[0],1)}
sub asin {my $z = $_[0]; atan2($z, sqrt(1-$z*$z))}
sub asind {asin($_[0])*180/$pi}

sub segnumber
{
 my $s = shift;
 my $zeros = "";
 if($s < 9) { $zeros = "0"; }
 my $segment_n = $s + 1;
 return "$zeros$segment_n";
}

sub pos_t
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta     = 90 + $starting_theta[$l] + $s*$dtheta[$l] ;
 my $x         = sprintf("%.3f",$R*cos(rad($theta)));
 my $y         = sprintf("%.3f",$R*sin(rad($theta)));
# return "$x*mm $y*mm";
 return "$0*mm $0*mm";
}

sub rot
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta_rot = $starting_theta[$l] - $s*$Dtheta;
# if($l == 2) {$theta_rot -= 20;}

 return "0*deg 0*deg $theta_rot*deg";
}

# BMT is a Tube. SLs are Tubes inside BMT

my $bmt_ir = 100;
my $bmt_or = 250;
my $bmt_dz = 341;

my @SL_ir = (109, 119, 169, 179, 229, 239);
my @SL_or = (114, 124, 174, 184, 234, 244);
my $SL_dz = 341;

make_bmt();
make_sl(1);
make_sl(2);
make_sl(3);
make_sl(4);
make_sl(5);
make_sl(6);

sub make_bmt
{
 $detector{"name"}        = $envelope;
 $detector{"mother"}      = "root";
 $detector{"description"} = "Barrel Micromegas Vertex Tracker";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$bmt_ir*mm $bmt_or*mm $bmt_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);

}

for(my $l = 0; $l < 6; $l++)
{
    my $layer_no       = $l + 1;
    
    place_epoxy($l);
    place_pcboard($l);
    place_strips($l);
    place_gas1($l);
    place_mesh($l);
    place_gas2($l);
    place_drift($l);
    
}

sub make_sl
{

 my $slnumber = shift;
 my $slindex  = $slnumber - 1;

 $detector{"name"}        = "SL2_$slnumber";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "Super Layer $slnumber";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$SL_ir[$slindex]*mm $SL_or[$slindex]*mm $SL_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);

}


sub place_epoxy
{
    my $l    = shift;
    my $layer_no       = $l + 1;
    my $r          = $radius[$l] + $Epoxy_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Epoxy_V_Layer";
	$descriptio = "Epoxy V, Layer $layer_no, ";
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_Epoxy_W_Layer";
	$descriptio = "Epoxy W, Layer $layer_no, ";
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l];
	my $PRMax     = $radius[$l] + $Epoxy_Width;
	my $PDz       = $Dz_halflength[$l]-$Epoxy_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      = "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $epoxy_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $epoxy_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);

    } 
}

sub place_pcboard
{
    my $l    = shift;
    my $layer_no       = $l + 1; 
    my $r          = $radius[$l] + $Epoxy_Width + $PCB_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_PCB_V_Layer";
	$descriptio = "PCB V, Layer $layer_no, ";
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_PCB_W_Layer";
	$descriptio = "PCB W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
	my $snumber   = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width;
	my $PDz       = $Dz_halflength[$l]-$PCB_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      =  "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $pcboard_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $pcboard_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);

    } 
}


sub place_strips
{
    my $l    = shift;
    my $layer_no       = $l + 1;
    my $r          = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Strips_V_Layer";
	$descriptio = "Strips V, Layer $layer_no, ";
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_Strips_W_Layer";
	$descriptio = "Strips W, Layer $layer_no, ";
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width + $PCB_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width;
	my $PDz       = $Dz_halflength[$l]-$Strips_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      =  "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $strips_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $strips_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);
	
    } 
}


sub place_gas1
{
    my $l    = shift;
    my $layer_no       = $l + 1;
    my $r  = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Gas1_V_Layer";
	$descriptio = "Gas1 V, Layer $layer_no, ";
    }
        if(($l % 2) == 1)
    {
	$vname      = "BMT_Gas1_W_Layer";
	$descriptio = "Gas1 W, Layer $layer_no, ";
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width;
	my $PDz       = $Dz_halflength[$l]-$Gas1_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      =  "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $gas_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $gas_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);

    } 
}


sub place_mesh
{
    my $l    = shift;
    my $layer_no = $l + 1;
    my $r = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Mesh_V_Layer";
	$descriptio = "Mesh V, Layer $layer_no, ";
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_Mesh_W_Layer";
	$descriptio = "Mesh W, Layer $layer_no, ";
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width;
	my $PDz       = $Dz_halflength[$l]-$Mesh_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      =  "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $mesh_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $mesh_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);

    } 
}


sub place_gas2
{
    my $l    = shift;
    my $layer_no = $l + 1;
    my $r = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width + $Gas2_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;
    my $type = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Gas2_V_Layer";
	$descriptio = "Gas2 V, Layer $layer_no, ";
	$type = 1;
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_Gas2_W_Layer";
	$descriptio = "Gas2 W, Layer $layer_no, ";
	$type = 2;
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width + $Gas2_Width;
	my $PDz       = $Dz_halflength[$l]-$Gas2_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      = "SL2_$layer_no";
        $detector{"description"} = "$descriptio  Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $gas_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    =  $gas_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "BMT";
        $detector{"hit_type"}    = "BMT";
        $detector{"identifiers"} ="superlayer manual $layer_no type manual $type segment manual $detector{'ncopy'} strip manual 1";

        print_det(\%detector, $file);

    } 
}


sub place_drift
{
    my $l = shift;
    my $layer_no = $l + 1;
    my $r = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width + $Gas2_Width + $Drift_Width / 2.0;
    my $vname      = 0;
    my $descriptio = 0;

    if(($l % 2) == 0)
    {
	$vname      = "BMT_Drift_V_Layer";
	$descriptio = "Drift V, Layer $layer_no, ";
    }
    if(($l % 2) == 1)
    {
	$vname      = "BMT_Drift_W_Layer";
	$descriptio = "Drift W, Layer $layer_no, ";
    }
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
	# names
        my $snumber     = segnumber($s);
	my $z         = $starting_point[$l] + $Dz_halflength[$l];
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width + $Gas2_Width;
	my $PRMax     = $radius[$l] + $Epoxy_Width + $PCB_Width + $Strips_Width + $Gas1_Width + $Mesh_Width + $Gas2_Width + $Drift_Width;
	my $PDz       = $Dz_halflength[$l]-$Drift_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta[$l];

        $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
        $detector{"mother"}      = "SL2_$layer_no";
        $detector{"description"} = "$descriptio Segment $snumber";
        $detector{"pos"}         = "$tpos $z*mm";
        $detector{"rotation"}    = rot($r, $l, $s);
        $detector{"color"}       = $drift_color;
        $detector{"type"}        = "Tube";
        $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
        $detector{"material"}    = $drift_material;
        $detector{"mfield"}      = "no";
        $detector{"ncopy"}       = $s + 1;
        $detector{"pMany"}       = 1;
        $detector{"exist"}       = 1;
        $detector{"visible"}     = 1;
        $detector{"style"}       = 1;
        $detector{"sensitivity"} = "no";
        $detector{"hit_type"}    = "";
        $detector{"identifiers"} = "";

        print_det(\%detector, $file);
	
    } 
}


