#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

use geo qw($pi);

# All dimensions in mm

my $envelope = 'FMT';
my $file     = 'FMT.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();          # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

# General description
#my @radius             = ( 34.100 , 34.100 ,  34.100 );  # Transverse Position Start of Modules
my @radius             = ( 10.000 , 10.000 ,  10.000 );  # Transverse Position Start of Modules
my @starting_point     = (350.000 , 370.000 , 390.000 );  # Z starting_point of the sensors

my $nsegments          = 1;                # Number of segments in each layer
# my $dtheta             = 360.0/$nsegments; # Delta theta
my $dtheta             = 60.0;
my $starting_theta     = $dtheta/2.0;      # Starting angle of the 1st segment


#######################
# Dimensions (in mm):
# Each segment is a Tube
#######################

# For N=6:
#my $Epoxy_RMin       = 19.100;   # Epoxy inner radius
my $Epoxy_RMin       = 5.000;   # Epoxy inner radius
my $Epoxy_RMax       = 280.000;  # Epoxy outer radius
my $Epoxy_Dz         = 0.300/2.; # Epoxy half width (in z)

#my $PCB_RMin         = 19.100;    # PCB inner radius
my $PCB_RMin         = 5.000;    # PCB inner radius
my $PCB_RMax         = 280.000;   # PCB outer radius
my $PCB_Dz           = 0.100/2.;  # PCB half width (in z)

#my $Strips_RMin      = 24.100;    # Strips inner radius
my $Strips_RMin      = 6.000;    # Strips inner radius
my $Strips_RMax      = 275.000;   # Strips outer radius
my $Strips_Dz        = 0.015/2.;  # Strips half width (in z)

#my $Gas1_RMin        = 24.100;    # Gas1 inner radius (in amplification gap)
my $Gas1_RMin        = 6.000;    # Gas1 inner radius (in amplification gap)
my $Gas1_RMax        = 275.000;   # Gas1 outer radius
my $Gas1_Dz          = 0.128/2.;  # Gas1 half width (in z)

#my $Mesh_RMin        = 24.100;    # Mesh inner radius
my $Mesh_RMin        = 6.000;    # Mesh inner radius
my $Mesh_RMax        = 275.000;   # Mesh outer radius
my $Mesh_Dz          = 0.030/2.;  # Mesh half width (in z)
#my $HolesMeshRadius  = 0.020875;     # radius of mesh holes
#my $HolesMeshPitch   = 0.050;     # pitch of holes in the mesh
my $HolesMeshRadius  = 0.01749;     # radius of mesh holes
my $HolesMeshPitch   = 0.050;     # pitch of holes in the mesh

#my $Gas2_RMin        = 24.100;    # Gas2 inner radius (in drift gap)
my $Gas2_RMin        = 6.000;    # Gas2 inner radius (in drift gap)
my $Gas2_RMax        = 275.000;   # Gas2 outer radius
my $Gas2_Dz          = 5.350/2.;  # Gas2 half width (in z)

#my $Drift_RMin       = 24.100;    # Drift inner radius (electrode)
my $Drift_RMin       = 6.000;    # Drift inner radius (electrode)
my $Drift_RMax       = 275.000;   # Drift outer radius
my $Drift_Dz         = 0.100/2.;  # Drift half width (in z)
#my $Drift_Dz         = 0.030/2.;  # Drift half width (in z)
my $HolesDriftRadius  = 0.020875;     # radius of drift mesh holes
my $HolesDriftPitch   = 0.100;     # pitch of holes in the drift mesh
#my $HolesDriftRadius  = 0.01749;     # radius of drift mesh holes
#my $HolesDriftPitch   = 0.050;     # pitch of holes in the drift mesh

# G4 materials
my $epoxy_material   = 'Epoxy';
my $pcboard_material = 'Epoxy';
my $strips_material  = 'MMStrips';
my $gas_material     = 'MMGas';
my $mesh_material    = 'MMMesh';
my $drift_material   = 'MMMylar';


# G4 colors
my $epoxy_color      = 'e200e1';
#my $pcboard_color    = '0000ff';
my $pcboard_color    = '0000ff';
my $strips_color     = '353540';
my $gas_color        = 'e10000';
my $mesh_color       = '252020';
my $drift_color      = 'fff600';
#my $drift_color      = '252020';

$pi = 3.141592653589793238;
#sub tan { sin($_[0]) / cos($_[0])  }
#sub rad { $_[0]*$pi/180.0  }
#sub atan {atan2($_[0],1)}
#sub asin {my $z = $_[0]; atan2($z, sqrt(1-$z*$z))}
#sub asind {asin($_[0])*180/$pi}

sub segnumber
{
    my $s = shift;
    my $zeros = "";
    if($s < 9) { $zeros = "0"; }
    my $segment_n = $s + 1;
    return "$zeros$segment_n"; 
}

sub pos_t
{
    my $R = shift;
    my $l = shift;
    my $s = shift;
    
 #   my $theta     = $starting_theta + $s*$dtheta ;
    my $theta     = $s*$dtheta ;
    my $x         = sprintf("%.3f", $R*cos(rad($theta)));
    my $y         = sprintf("%.3f", $R*sin(rad($theta)));
    return "$x*mm $y*mm";
}

sub rot
{
    my $R = shift;
    my $l = shift;
    my $s = shift;
    
    #my $theta_rot = $starting_theta + $s*$dtheta - 90;
    my $theta_rot = $starting_theta - $s*$dtheta;
    
#    return "90*deg $theta_rot*deg 0*deg";
    return "0*deg 0*deg $theta_rot*deg";
}


# FMT is a Tube containing all SLs

#my $fmt_ir = 18.5;
my $fmt_ir = 5.0;
my $fmt_or = 300.0;
my $fmt_dz = ($starting_point[2] - $starting_point[0])/2.0 + 8.0;

my $fmt_starting = $starting_point[1];
make_fmt();

sub make_fmt
{
    my $zpos      = $fmt_starting;
    
    $detector{"name"}        = $envelope;
    $detector{"mother"}      = "root";
    $detector{"description"} = "Forward Micromegas Vertex Tracker";
    
    # names
    my $name        = $envelope;
    my $mother      = "root";
    my $description = "Forward Micromegas Vertex Tracker";
    $detector{"pos"}         = "0*mm 0*mm $zpos*mm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = "aaaaff";
    $detector{"type"}        = "Tube";
    $detector{"dimensions"}  = "$fmt_ir*mm $fmt_or*mm $fmt_dz*mm 0*deg 360*deg";
    $detector{"material"}    = "Air";
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 0;
    $detector{"style"}       = 0;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";
    
    print_det(\%detector, $file);
}

for(my $l = 0; $l < 1; $l++) 
{
    my $layer_no       = $l + 1;
    
    place_drift($l, 1);
#    place_holesindrift($l, 1);
    place_gas2($l, 1);
    place_mesh($l, 1);
    place_holesinmesh($l, 1);
    place_gas1($l, 1);
    place_strips($l, 1);
    place_pcboard($l, 1);
    place_epoxy($l);
#    place_pcboard($l, 2);
#    place_strips($l, 2);
#    place_gas1($l, 2);
#    place_mesh($l, 2);
#    place_gas2($l, 2);
#    place_drift($l, 2);
}

sub place_epoxy
{
    my $l    = shift; 
    my $layer_no       = $l + 1;
    
    my $z          = - $fmt_starting + $starting_point[$l];
    my $vname      = "FMT_Epoxy";
    my $descriptio = "Epoxy, Layer $layer_no, ";
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r         = $radius[$l] - $Epoxy_RMin; # + ($Epoxy_RMax-$Epoxy_RMin)/2.0;
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $Epoxy_RMin;
	my $PRMax     = $Epoxy_RMax;
	my $PDz       = $Epoxy_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $epoxy_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $epoxy_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    } 
}

sub place_pcboard
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;

    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz;
	$vname       = "FMT_PCBoard_V_L";
	$descriptio  = "PC Board V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz;
	$vname       = "FMT_PCBoard_W_L";
	$descriptio  = "PC Board W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber   = segnumber($s);
	my $r         = $radius[$l] - $Epoxy_RMin; # + ($PCB_RMax-$PCB_RMin) / 2.0;
	my $tpos      = pos_t($r, $l, $s);
	my $PRMin     = $PCB_RMin;
	my $PRMax     = $PCB_RMax;
	my $PDz       = $PCB_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $pcboard_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $pcboard_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    }
}

sub place_strips
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;

    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz;
	$vname       = "FMT_Strips_V_L";
	$descriptio  = "Strips V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz;
	$vname       = "FMT_Strips_W_L";
	$descriptio  = "Strips W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r          = $radius[$l] - $Epoxy_RMin; # + ($Strips_RMax-$Strips_RMin) / 2.0;
	my $tpos       = pos_t($r, $l, $s);
	my $PRMin     = $Strips_RMin;
	my $PRMax     = $Strips_RMax;
	my $PDz       = $Strips_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $strips_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $strips_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    }
}

sub place_gas1
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz;
	$vname       = "FMT_Gas1_V_L";
	$descriptio  = "Gas1 V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz;
	$vname       = "FMT_Gas1_W_L";
	$descriptio  = "Gas1 W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r          = $radius[$l] - $Epoxy_RMin; # + ($Gas1_RMax-$Gas1_RMin) / 2.0;
	my $tpos       = pos_t($r, $l, $s);
	my $PRMin     = $Gas1_RMin;
	my $PRMax     = $Gas1_RMax;
	my $PDz       = $Gas1_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $gas_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $gas_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    }
}

sub place_mesh
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz;
	$vname       = "FMT_Mesh_V_L";
	$descriptio  = "Mesh V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz;
	$vname       = "FMT_Mesh_W_L";
	$descriptio  = "Mesh W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r          = $radius[$l] - $Epoxy_RMin; # + ($Mesh_RMax-$Mesh_RMin) / 2.0;
	my $tpos       = pos_t($r, $l, $s);
	my $PRMin     = $Mesh_RMin;
	my $PRMax     = $Mesh_RMax;
	my $PDz       = $Mesh_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $mesh_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $mesh_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    }
}

sub place_holesinmesh
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $vname      = 0;
    my $vmothername = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$vname       = "FMT_HolesMesh_V_L";
	$vmothername = "FMT_Mesh_V_L";
	$descriptio  = "HolesMesh V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$vname       = "FMT_HolesMesh_W_L";
	$vmothername = "FMT_Mesh_W_L";
	$descriptio  = "HolesMesh W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $Nmaxholesperline = $Mesh_RMax/$HolesMeshPitch;
#	my $Nmaxholesperline = 10;
	my $holenumber = 0;
	for(my $holexnumber = 0; $holexnumber <$Nmaxholesperline  ; $holexnumber++)
	{
	    for(my $holeynumber = 0; $holeynumber <$Nmaxholesperline  ; $holeynumber++)
	    {
		my $xpos = $HolesMeshPitch*$holexnumber;
		my $ypos = $HolesMeshPitch*$holeynumber;
		my $PRMin     = 0.0;
		my $PRMax     = $HolesMeshRadius;
		my $PDz       = $Mesh_Dz;
		my $PSPhi     = 0.0;
		my $PDPhi     = 360.0;
		if($xpos*$xpos+$ypos*$ypos<($Mesh_RMax-175.)*($Mesh_RMax-175.) && $xpos*$xpos+$ypos*$ypos>($Mesh_RMin+40.)*($Mesh_RMin+40.) && $ypos<1.0*$xpos  && $ypos>0.3*$xpos && $s == 0)
#		if($xpos*$xpos+$ypos*$ypos<($Mesh_RMax-150.)*($Mesh_RMax-150.) && $xpos*$xpos+$ypos*$ypos>($Mesh_RMin+20.)*($Mesh_RMin+20.) && $ypos<1.3*$xpos  && $ypos>0.1*$xpos && $s == 0)
		{
		    $holenumber = $holenumber +1;
		    $detector{"name"}        = "$vname$layer_no\_Segment$snumber\_Hole$holenumber";
		    $detector{"mother"}      = "$vmothername$layer_no\_Segment$snumber";
#	$detector{"description"} = "$descriptio Segment $snumber Hole $holenumber";
		    $detector{"description"} = "$descriptio Segment $snumber";
		    $detector{"pos"}         = "$xpos*mm $ypos*mm 0*mm";
		    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
		    $detector{"color"}       = $gas_color;
		    $detector{"type"}        = "Tube";
		    $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
		    $detector{"material"}    = $gas_material;
		    $detector{"mfield"}      = "no";
		    $detector{"ncopy"}       = 10*$s + $holenumber + 1;
		    $detector{"pMany"}       = 1;
		    $detector{"exist"}       = 1;
		    $detector{"visible"}     = 1;
		    $detector{"style"}       = 1;
		    $detector{"sensitivity"} = "no";
		    $detector{"hit_type"}    = "";
		    $detector{"identifiers"} = "";
		    
		    print_det(\%detector, $file);
		}
	    }
	}
    }
}

sub place_gas2
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz*2. - $Gas2_Dz;
	$vname       = "FMT_Gas2_V_L";
	$descriptio  = "Gas2 V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz*2. + $Gas2_Dz;
	$vname       = "FMT_Gas2_W_L";
	$descriptio  = "Gas2 W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r          = $radius[$l] - $Epoxy_RMin; # + ($Gas2_RMax-$Gas2_RMin) / 2.0;
	my $tpos       = pos_t($r, $l, $s);
	my $PRMin     = $Gas2_RMin;
	my $PRMax     = $Gas2_RMax;
	my $PDz       = $Gas2_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "$descriptio  Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $gas_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    =  $gas_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "FMT";
	$detector{"hit_type"}    = "FMT";
	$detector{"identifiers"} ="superlayer manual $layer_no type manual $type segment manual $detector{'ncopy'} strip manual 1";
	
	print_det(\%detector, $file);
    }
}

sub place_drift
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$z           =   - $fmt_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz*2. - $Gas2_Dz*2. - $Drift_Dz;
	$vname       = "FMT_Drift_V_L";
	$descriptio  = "Drift V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$z           =   - $fmt_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz*2. + $Gas2_Dz*2. + $Drift_Dz;
	$vname       = "FMT_Drift_W_L";
	$descriptio  = "Drift W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $r          = $radius[$l] - $Epoxy_RMin; # + ($Drift_RMax-$Drift_RMin) / 2.0;
	my $tpos       = pos_t($r, $l, $s);
	my $PRMin     = $Drift_RMin;
	my $PRMax     = $Drift_RMax;
	my $PDz       = $Drift_Dz;
	my $PSPhi     = 0.0; # to be defined, in degres
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$snumber";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $snumber";
	$detector{"pos"}         = "$tpos $z*mm";
	$detector{"rotation"}    = rot($r, $l, $s);
	$detector{"color"}       = $drift_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $drift_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
    }
}

sub place_holesindrift
{
    my $l    = shift; 
    my $type = shift;
    my $layer_no       = $l + 1;
    my $vname      = 0;
    my $vmothername = 0;
    my $descriptio = 0;
    
    if($type == 1)
    {
	$vname       = "FMT_HolesDrift_V_L";
	$vmothername = "FMT_Drift_V_L";
	$descriptio  = "HolesDrift V, Layer $layer_no, ";
    }
    
    if($type == 2) 
    {
	$vname       = "FMT_HolesDrift_W_L";
	$vmothername = "FMT_Drift_W_L";
	$descriptio  = "HolesDrift W, Layer $layer_no, ";
    }
    
    for(my $s = 0; $s < $nsegments ; $s++)
    {
	# names
	my $snumber     = segnumber($s);
	my $Nmaxholesperline = $Drift_RMax/$HolesDriftPitch;
#	my $Nmaxholesperline = 10;
	my $holenumber = 0;
	for(my $holexnumber = 0; $holexnumber <$Nmaxholesperline  ; $holexnumber++)
	{
	    for(my $holeynumber = 0; $holeynumber <$Nmaxholesperline  ; $holeynumber++)
	    {
		my $xpos = $HolesDriftPitch*$holexnumber;
		my $ypos = $HolesDriftPitch*$holeynumber;
		my $PRMin     = 0.0;
		my $PRMax     = $HolesDriftRadius;
		my $PDz       = $Drift_Dz;
		my $PSPhi     = 0.0;
		my $PDPhi     = 360.0;
#		if($xpos*$xpos+$ypos*$ypos<$Mesh_RMax*$Mesh_RMax && $xpos*$xpos+$ypos*$ypos>$Mesh_RMin*$Mesh_RMin && $ypos/$xpos<1.73)
#		if($xpos*$xpos+$ypos*$ypos<($Mesh_RMax-175.)*($Mesh_RMax-175.) && $xpos*$xpos+$ypos*$ypos>($Mesh_RMin+40.)*($Mesh_RMin+40.) && $ypos<1.73*$xpos && $s == 0)
		if($xpos*$xpos+$ypos*$ypos<($Drift_RMax-175.)*($Drift_RMax-175.) && $xpos*$xpos+$ypos*$ypos>($Drift_RMin+40.)*($Drift_RMin+40.) && $ypos<1.0*$xpos  && $ypos>0.3*$xpos && $s == 0)
		{
		    $holenumber = $holenumber +1;
		    $detector{"name"}        = "$vname$layer_no\_Segment$snumber\_Hole$holenumber";
		    $detector{"mother"}      = "$vmothername$layer_no\_Segment$snumber";
#	$detector{"description"} = "$descriptio Segment $snumber Hole $holenumber";
		    $detector{"description"} = "$descriptio Segment $snumber";
		    $detector{"pos"}         = "$xpos*mm $ypos*mm 0*mm";
		    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
		    $detector{"color"}       = $gas_color;
		    $detector{"type"}        = "Tube";
		    $detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
		    $detector{"material"}    = $gas_material;
		    $detector{"mfield"}      = "no";
		    $detector{"ncopy"}       = 10*$s + $holenumber + 1;
		    $detector{"pMany"}       = 1;
		    $detector{"exist"}       = 1;
		    $detector{"visible"}     = 1;
		    $detector{"style"}       = 1;
		    $detector{"sensitivity"} = "no";
		    $detector{"hit_type"}    = "";
		    $detector{"identifiers"} = "";
		    
		    print_det(\%detector, $file);
		}
	    }
	}
    }
}
