#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

use geo qw($pi);

# All dimensions in mm

my $envelope = 'FTM';
my $file     = 'FTM.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();          # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

# General description
my @starting_point     = (1774.000 , 1794.000);  # Z starting_point of the sensors
my $dtheta             = 179.0;
my $nsegments          = 2;

#######################
# Dimensions (in mm):
# Each segment is a Tube
#######################
my $Epoxy_RMin       = 80.000;   # Epoxy inner radius
my $Epoxy_RMax       = 160.000;  # Epoxy outer radius
my $Epoxy_Dz         = 0.300/2.; # Epoxy half width (in z)

my $PCB_RMin         = 80.000;    # PCB inner radius
my $PCB_RMax         = 160.000;   # PCB outer radius
my $PCB_Dz           = 0.100/2.;  # PCB half width (in z)

my $Strips_RMin      = 80.000;    # Strips inner radius
my $Strips_RMax      = 160.000;   # Strips outer radius
my $Strips_Dz        = 0.015/2.;  # Strips half width (in z)

my $Gas1_RMin        = 80.000;    # Gas1 inner radius (in amplification gap)
my $Gas1_RMax        = 160.000;   # Gas1 outer radius
my $Gas1_Dz          = 0.128/2.;  # Gas1 half width (in z)

my $Mesh_RMin        = 80.000;    # Mesh inner radius
my $Mesh_RMax        = 160.000;   # Mesh outer radius
my $Mesh_Dz          = 0.030/2.;  # Mesh half width (in z)

my $Gas2_RMin        = 80.000;    # Gas2 inner radius (in drift gap)
my $Gas2_RMax        = 160.000;   # Gas2 outer radius
my $Gas2_Dz          = 5.350/2.;  # Gas2 half width (in z)

my $Drift_RMin       = 80.000;    # Drift inner radius (electrode)
my $Drift_RMax       = 160.000;   # Drift outer radius
my $Drift_Dz         = 0.100/2.;  # Drift half width (in z)


# G4 materials
my $epoxy_material   = 'Epoxy';
my $pcboard_material = 'Epoxy';
my $strips_material  = 'MMStrips';
my $gas_material     = 'MMGas';
my $mesh_material    = 'MMMesh';
my $drift_material   = 'MMMylar';


# G4 colors
my $epoxy_color      = 'e200e1';
my $pcboard_color    = '0000ff';
my $strips_color     = '353540';
my $gas_color        = 'e10000';
my $mesh_color       = '252020';
my $drift_color      = 'fff600';

$pi = 3.141592653589793238;


# FTM is a Tube containing all SLs

my $ftm_ir = 5.0;
my $ftm_or = 300.0;
my $ftm_dz = ($starting_point[1] - $starting_point[0])/2.0 + 17.0;

my $ftm_starting = $starting_point[1];
make_ftm();

sub make_ftm
{
    my $zpos      = $ftm_starting;
    
    $detector{"name"}        = $envelope;
    $detector{"mother"}      = "root";
    $detector{"description"} = "Forward Tagger Micromegas";
    
    # names
    my $name        = $envelope;
    my $mother      = "root";
    my $description = "Forward Tagger Micromegas";
    $detector{"pos"}         = "0*mm 0*mm $zpos*mm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = "aaaaff";
    $detector{"type"}        = "Tube";
    $detector{"dimensions"}  = "$ftm_ir*mm $ftm_or*mm $ftm_dz*mm 0*deg 360*deg";
    $detector{"material"}    = "Air";
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 0;
    $detector{"style"}       = 0;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";
    
    print_det(\%detector, $file);
}

for(my $l = 0; $l < 2; $l++) 
{
    my $layer_no       = $l + 1;

    for(my $s = 0; $s < $nsegments ; $s++)
    {    
       place_epoxy($l,$s);

# X layer type
       place_pcboard($l,1,$s);
       place_strips($l,1,$s);
       place_gas1($l,1,$s);
       place_mesh($l,1,$s);
       place_gas2($l,1,$s);
       place_drift($l,1,$s);

# Y layer type
       place_pcboard($l,2,$s);
       place_strips($l,2,$s);
       place_gas1($l,2,$s);
       place_mesh($l,2,$s);
       place_gas2($l,2,$s);
       place_drift($l,2,$s);
    }
}


sub place_epoxy
{
    my $l    = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    
    my $z          = - $ftm_starting + $starting_point[$l];
    my $vname      = "FTM_Epoxy";
    my $descriptio = "Epoxy, Layer $layer_no, ";
    
	# names
	my $r         = 0.000;
	my $PRMin     = $Epoxy_RMin;
	my $PRMax     = $Epoxy_RMax;
	my $PDz       = $Epoxy_Dz;
	my $PSPhi     = (2*$s-1)*90;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $epoxy_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $epoxy_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file); 
}

sub place_pcboard
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no   = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;

    if($type == 1)
    {
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz;
	$vname       = "FTM_PCBoard_X_L";
	$descriptio  = "PC Board X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }
    
    if($type == 2) 
    {
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz;
	$vname       = "FTM_PCBoard_Y_L";
	$descriptio  = "PC Board Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }
    
	# names
	my $r         = 0.000;
	my $PRMin     = $PCB_RMin;
	my $PRMax     = $PCB_RMax;
	my $PDz       = $PCB_Dz;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $pcboard_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $pcboard_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}

sub place_strips
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;

    if($type == 1)
    {
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz;
	$vname       = "FTM_Strips_X_L";
	$descriptio  = "Strips X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }
    
    if($type == 2) 
    {
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz;
	$vname       = "FTM_Strips_Y_L";
	$descriptio  = "Strips Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }
    
	# names
	my $r         = 0.000;
	my $PRMin     = $Strips_RMin;
	my $PRMax     = $Strips_RMax;
	my $PDz       = $Strips_Dz;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $strips_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $strips_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}

sub place_gas1
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;
    
    if($type == 1)
    {
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz;
	$vname       = "FTM_Gas1_X_L";
	$descriptio  = "Gas1 X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }
    
    if($type == 2) 
    {
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz;
	$vname       = "FTM_Gas1_Y_L";
	$descriptio  = "Gas1 Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }
    
	# names
	my $r         = 0.000;
	my $PRMin     = $Gas1_RMin;
	my $PRMax     = $Gas1_RMax;
	my $PDz       = $Gas1_Dz;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $gas_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $gas_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}

sub place_mesh
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;
  
    if($type == 1)
    {  
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz;
	$vname       = "FTM_Mesh_X_L";
	$descriptio  = "Mesh X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }  

    if($type == 2)
    {  
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz;
	$vname       = "FTM_Mesh_Y_L";
	$descriptio  = "Mesh Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }    

	# names
	my $r         = 0.000;
	my $PRMin     = $Mesh_RMin;
	my $PRMax     = $Mesh_RMax;
	my $PDz       = $Mesh_Dz;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $mesh_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $mesh_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}


sub place_gas2
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;
    
    if($type == 1)
    {
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz*2. - $Gas2_Dz;
	$vname       = "FTM_Gas2_X_L";
	$descriptio  = "Gas2 X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }
    
    if($type == 2) 
    {
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz*2. + $Gas2_Dz;
	$vname       = "FTM_Gas2_Y_L";
	$descriptio  = "Gas2 Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }
    
	# names
	my $r         = 0.000;
	my $PRMin     = $Gas2_RMin;
	my $PRMax     = $Gas2_RMax;
	my $PDz       = $Gas2_Dz;
	my $PDPhi     = $dtheta;

	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $gas_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    =  $gas_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "FTM";
	$detector{"hit_type"}    = "FTM";
	$detector{"identifiers"} ="superlayer manual $layer_no type manual $type segment manual $detector{'ncopy'} strip manual 1";
	
	print_det(\%detector, $file);
}


sub place_drift
{
    my $l    = shift; 
    my $type = shift;
    my $s    = shift;
    my $layer_no       = $l + 1;
    my $seg_no         = $s + 1;
    my $z          = 0;
    my $vname      = 0;
    my $descriptio = 0;
    my $PSPhi     = 0.0;
      
    
    if($type == 1)
    {
	$z           =   - $ftm_starting + $starting_point[$l] - $Epoxy_Dz - $PCB_Dz*2. - $Strips_Dz*2. - $Gas1_Dz*2. - $Mesh_Dz*2. - $Gas2_Dz*2. - $Drift_Dz;
	$vname       = "FTM_Drift_X_L";
	$descriptio  = "Drift X, Layer $layer_no, ";
        $PSPhi 	     = (2*$s - 1)*90;
    }

    if($type == 2)
    {
	$z           =   - $ftm_starting + $starting_point[$l] + $Epoxy_Dz + $PCB_Dz*2. + $Strips_Dz*2. + $Gas1_Dz*2. + $Mesh_Dz*2. + $Gas2_Dz*2. + $Drift_Dz;
	$vname       = "FTM_Drift_Y_L";
	$descriptio  = "Drift Y, Layer $layer_no, ";
        $PSPhi 	     = 180.000*$s;
    }

	# names
	my $r         = 0.000;
	my $PRMin     = $Drift_RMin;
	my $PRMax     = $Drift_RMax;
	my $PDz       = $Drift_Dz;
	my $PDPhi     = $dtheta;
	
	$detector{"name"}        = "$vname$layer_no\_Segment$seg_no";
	$detector{"mother"}      =  $envelope;
	$detector{"description"} = "$descriptio Segment $seg_no";
	$detector{"pos"}         = "0.000*mm 0.000*mm $z*mm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = $drift_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$PRMin*mm $PRMax*mm $PDz*mm $PSPhi*deg $PDPhi*deg";
	$detector{"material"}    = $drift_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = $s + 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}


