#!/usr/bin/perl -w

# Written by Alexander Zenon Piaseczny

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);

my $envelope      = 'PCAL';
my $file          = 'PCAL.txt';
my $rmin          = 1;
my $rmax          = 1000000;
my %detector      = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

use Getopt::Long;
use Math::Trig;

my $inches   = 2.54;
my $dlead = 2.2; #millimeters (mm)                                            
my $dscint = 10.0; #millimeters (mm)

#soh cah toa
#sin(theta) = opp/hyp; cos(theta) = adj/hyp; tan(theta) = opp/adj;

#	 |\
#	 | \
#	 |  \
#	 |   \
#	 |    \
#    opp |     \ hyp
#	 |      \	
#	 |       \ 
#	 |        \
#	 |__  theta\
#           adj

#Dimensions of the CLAS12_Pre-Shower_Calorimeter are located
#on the website: http://clasweb.jlab.org/wiki/index.php/CLAS12_Pre-shower_calorimeter

#However, the dimensions given do not produce the angles given on the website above.

# PCAL symmetric geometry:

#		        			       $x_length_pcal = 394.32*cm
#		   B				      A_______________________C _ 
# 	          /|\              		       \          |          /  |        
#	         / g \                		        \         |         /   |        
#	        /  |  \				         \        |        /    |
#	       /   |   \				  \       |       /     |       
#	      /    |    \				   \      |      /      | --> $height_pcal 
#	     /	   |	 \ --> #$width_pcal = 432.87*cm <-- \     |     /       |               
#	    /	   | 	  \                                  \    |    /        | 
#	   /	   |       \	               		      \   |   /         |
#	  /	   |	    \			               \  |  /          |
#	 /	   |	     \  				\ | / 	        |  
#     A	/a_________|_________a\ C			      	 \|/	        |
#								  B               	
#g = gamma angle (aka "apex angle")
#a = alpha angle
# 									| y
#Looking at one PCAL sector from the target;                            |    coordinate axes; origin is at the geometric
#z-axis is pointing out of the screen                            _______|    center of the triangle
#           							 x
	       	               	         
my $width_pcal = 4328.7;    #millimeters (mm) #Side AB,BC
my $x_length_pcal = 3943.2; #millimeters (mm) #Side AC

my $alpha_angle_rad = acos($x_length_pcal/(2*$width_pcal)); #produces an angle in radians 
my $alpha_angle_deg = $alpha_angle_rad*180/$pi;             #produces an angle in degrees

my $gamma_angle_deg = 180 - 2*$alpha_angle_deg; #produces an angle in degrees
my $gamma_angle_rad = $gamma_angle_deg*$pi/180; #produces an angle in radians

my $opp_triangle_alpha = $width_pcal*sin($alpha_angle_rad);
my $height_pcal = $opp_triangle_alpha; #millimeters (mm)

my $thickness_scint_strip = 45; #millimeters (mm)
my $number_of_strips = $opp_triangle_alpha/$thickness_scint_strip; #For U_view

my $tantheta = tan($alpha_angle_rad);

print "\n";
print "--------------------------------------------------------------------- \n";
print "The angles of a triangle given 3 sides \n";
print "\t\t\t\t\t		  B       a = alpha angle \n";
print "\t\t\t\t\t		 /g\\      g = gamma angle \n";
print "Side AB: $width_pcal mm\t\t\t\t        / | \\ \n";
print "Side BC: $width_pcal mm\t\t\t\t       /  |  \\ \n";
print "Side AC: $x_length_pcal mm\t\t\t\t      /   |   \\ \n";
print "\t\t\t\t\t\t     /    |    \\ \n";
print "Angle ABC: $gamma_angle_deg deg\t\t\t    /     |     \\  \n";
print "Angle BAC: $alpha_angle_deg deg\t\t\t   /     _|_     \\ \n";
print "Angle BCA: $alpha_angle_deg deg\t\t\tA /a____ |||_____a\\ C \n";
print "--------------------------------------------------------------------- \n";
print "\n";
print "The height of PCAL Mother volume is: $height_pcal mm \n"; 
print "The U scintillator has: $number_of_strips strips \n\n";

my $nlayers = 15.0;		                             
my $PCAL_angle = 25.0; # "rotation"                                      
my $total_thickness = ($nlayers)*($dscint+$dlead) - $dlead;   #15 scintillators layers, 14 lead layers

my $L0 = 5103.2;                                             # length of perpendicular line at origin of local PCAL 								       mother volume that passes through the CLAS6 target.
my $L1 = 7217.23;                                            # length of perpendicular line at origin of local PCAL 								       mother volume that passes through the CLAS12 target.
my $MUoffset = 5000.0;                                       # the CLAS12 target is at +5m in the gemc coordinates.
my @beamVertexCut=(0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001, 0.001);

# PCAL Mother Dimensions (Utilizing the type: "G4Trap")
my $pDx1mom   = 0.00000001;  #GEANT4 will not accept a dimension of 'zero'
my $pDx3mom   = 0.00000001;
my $pDx2mom   = $x_length_pcal/2;
my $pDx4mom   = $x_length_pcal/2;
my $pDy1mom   = $height_pcal/2;
my $pDy2mom   = $height_pcal/2;
my $pDzmom    = $total_thickness/2.0;
my $pThetamom = 0.0;
my $pPhimom   = 0.0;
my $pAlp1mom  = 0.0;
my $pAlp2mom  = 0.0;

my $PCAL_anglerad = $PCAL_angle*$pi/180;
my $dm = $L0 + (yplus($nlayers) + yminus($nlayers))/(2*tan($PCAL_anglerad));

my $pos1 = 480.478; #z global coordinate position of EC in gemc
my $pos2 = 1.524; #PCAL is to be placed behind EC by this amount

my $PCAL_centerx = 0; 
my $PCAL_centery = $dm*sin($PCAL_anglerad); 
my $PCAL_centerz = ($dm*cos($PCAL_anglerad) + ($L1 - $dm)/cos($PCAL_anglerad) - $MUoffset) - ($pos1 + $pos2);

sub make_mother_PCAL
{
	$detector{"name"}        = "PCAL";
	$detector{"mother"}      = "sector";
	$detector{"description"} = "Forward Calorimeter";
	$detector{"pos"}         = "${PCAL_centerx}*mm ${PCAL_centery}*mm ${PCAL_centerz}*mm";
	$detector{"rotation"}    = "$PCAL_angle*deg 0*deg 0*deg";	
	$detector{"color"}       = "ff1111"; # 33ffcc --> cyan
	$detector{"type"}        = "G4Trap";
	$detector{"dimensions"}  = "${pDzmom}*mm ${pThetamom}*deg ${pPhimom}*deg ${pDy1mom}*mm ${pDx1mom}*mm ${pDx2mom}*mm ${pAlp1mom}*deg ${pDy2mom}*mm ${pDx3mom}*mm ${pDx4mom}*mm ${pAlp2mom}*deg";
	$detector{"material"}    = "Air";
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 0;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";

	print_det(\%detector, $file);
}

#Lead LAYER	
sub make_lead
{
	for(my $i = 1.0; $i < $nlayers; $i++)
	{ 
		my $x0lead = 0.0; 
    		my $y0lead = $beamVertexCut[$i-2]/2;
    		my $z0lead = -$total_thickness/2 + ($i - 1)*($dscint + $dlead) + $dscint/2 + ($dscint+$dlead)/2;       

    		my $pDz = $dlead/2.0;
    		my $pDy1 = $pDy1mom;
    		my $pDy2 = $pDy2mom;
    		my $pDx1 = $pDx1mom;
    		my $pDx3 = $pDx3mom;
    		my $pDx2 = $pDx2mom;
    		my $pDx4 = $pDx4mom;
    		my $pTheta = $pThetamom;  
    		my $pPhi   = $pThetamom;  
    		my $pAlp1  = $pThetamom;  
    		my $pAlp2  = $pThetamom;  

    		$detector{"name"}        = "PCAL_Lead_Layer_${i}";
    		$detector{"mother"}      = "PCAL";
    		$detector{"description"} = "Forward Calorimeter lead layer ${i}";
    		$detector{"pos"}         = "${x0lead}*mm ${y0lead}*mm ${z0lead}*mm";
    		$detector{"rotation"}    = "0*deg 0*deg 0*deg";
    		$detector{"color"}       = "66ff33";
    		$detector{"type"}        = "G4Trap";
    		$detector{"dimensions"}  = "${pDz}*mm ${pTheta}*deg ${pPhi}*deg  ${pDy1}*mm ${pDx1}*mm  ${pDx2}*mm  ${pAlp1}*deg  ${pDy2}*mm ${pDx3}*mm  ${pDx4}*mm  ${pAlp2}*deg";
   		$detector{"material"}    = "Lead";
    		$detector{"mfield"}      = "no";
    		$detector{"ncopy"}       = $i;
    		$detector{"pMany"}       = 1;
    		$detector{"exist"}       = 1;
    		$detector{"visible"}     = 1;
    		$detector{"style"}       = 1;
    		$detector{"sensitivity"} = "no";
    		$detector{"hit_type"}    = "";
    		$detector{"identifiers"} = "";

    		print_det(\%detector, $file);
	}
}

#U-view Scintillator Strips
sub make_scintillator_U 
{
	my $view = 1;  #All U scintillator layers will be assigned as 1
	my $strip_no = 0;  

	for(my $k = 1.0; $k <= $nlayers; $k+=3)
	{
		for(my $j = 0.00000001; $j < $number_of_strips; $j++)  
		{	
			$strip_no++;  
			my $x_scint = 0.0;
			my $y_scint = -$pDy1mom + (($thickness_scint_strip)/2.0) + ($j*$thickness_scint_strip);
			my $z_scint = -$total_thickness/2 + ($k - 1)*($dscint + $dlead) + $dscint/2;
		
			my $pDz = $dscint/2.0;
    			my $pDy1 = ($thickness_scint_strip)/2.0;
    			my $pDy2 = ($thickness_scint_strip)/2.0;
    			my $pDx1 = $j*$thickness_scint_strip*tan($gamma_angle_rad/2.0);
    			my $pDx3 = $j*$thickness_scint_strip*tan($gamma_angle_rad/2.0);
   			my $pDx2 = ($j + 1)*$thickness_scint_strip*tan($gamma_angle_rad/2.0);
    			my $pDx4 = ($j + 1)*$thickness_scint_strip*tan($gamma_angle_rad/2.0);
    			my $pTheta = 0.0;
    			my $pPhi   = 0.0;
    			my $pAlp1  = 0.0;
    			my $pAlp2  = 0.0;	

			$detector{"name"}        = "U-view_strip ${strip_no}";
 			$detector{"mother"}      = "PCAL";
 			$detector{"description"} = "Forward Calorimeter scintillator layer ${j}";
 			$detector{"pos"}         = "${x_scint}*mm ${y_scint}*mm ${z_scint}*mm";
 			$detector{"rotation"}    = "0*deg 0*deg 0*deg";
 			$detector{"color"}       = "ff6633";
 			$detector{"type"}        = "G4Trap";
			$detector{"dimensions"}  = "${pDz}*mm ${pTheta}*deg ${pPhi}*deg ${pDy1}*mm ${pDx1}*mm  ${pDx2}*mm  ${pAlp1}*deg ${pDy2}*mm ${pDx3}*mm  ${pDx4}*mm  ${pAlp2}*deg";
			$detector{"material"}    = "Scintillator";
 			$detector{"mfield"}      = "no";
 			$detector{"ncopy"}       = $strip_no;
 			$detector{"pMany"}       = 1;
 			$detector{"exist"}       = 1;
 			$detector{"visible"}     = 1;
 			$detector{"style"}       = 0;
 			$detector{"sensitivity"} = "PCAL";
 			$detector{"hit_type"}    = "PCAL";
 			$detector{"identifiers"} = "sector ncopy 0 stack manual $k view manual $view strip manual $strip_no";

 			print_det(\%detector, $file);
		}
	}
}

#Symmetric geometry for the V,W strips:

#            ____________________        ___
#	   / |	                 | \	  |             		      
#	  /  |	   	         |  \ 	  | --> $thickness_scint_strip = 4.5*cm		               
#	 / 1 |	   	         | 2 \ 	  |			 
#     	/t___|___________________|___t\  _|_   
#								                 	
# t = theta angle							| y'
#Looking at one PCAL sector from the target;                            |    coordinate axes; origin is at the geometric
#z-axis is pointing out of the screen                            _______|    center of the trapezoid
#           							 x'
#Anti-Symmetric geometry for the V,W strips:

#            ____________________          ___
#	   / |	                 |   \	    |             		      
#	  /  |	   	         |    \     | --> $thickness_scint_strip = 4.5*cm		               
#	 / 1 |	   	         |  2  \    |			 
#     	/a___|___________________|_____g\  _|_   
#								                 	
#g = gamma angle (aka "apex angle")
#a = alpha angle

my $adj_triangle_alpha1 = $thickness_scint_strip/tan($alpha_angle_rad); #The cut for triangle 1 via alpha, anti-symmetric
my $adj_triangle_gamma2 = $thickness_scint_strip/tan($gamma_angle_rad); #The cut for triangle 2 via gamma, anti-symmetric

my $pDx1 = $width_pcal;
my $pDx2 = $width_pcal - ($adj_triangle_alpha1 + $adj_triangle_gamma2);

my $diff_btw_pDx1_and_pDx2 = $pDx1 - $pDx2;     #The difference between pDx1 and pDx2 in millimeters (mm)
my $symmetric_cuts = $diff_btw_pDx1_and_pDx2/2; #The symmetric cuts in millimeters (mm)

my $symmetric_angle_rad = atan($thickness_scint_strip/$symmetric_cuts); #The symmetric angle in radians
my $symmetric_angle_deg = $symmetric_angle_rad*180/$pi;                 #The symmetric angle in degrees 

#For the isoceles triangle whose dimensions are: pDx1 = pDx3 = width_pcal, pDx2 = pDx4 = 0.00000001 
my $height_v  = ($width_pcal/2)*tan($symmetric_angle_rad);
print "The height of the V and W scintillator is: $height_v mm, when the dimensions are: \n";
print "pDx1 = pDx3 = 4328.7 mm \n";
print "pDx2 = pDx4 = 0.00000001 mm \n";
my $number_of_strips_V_W = $height_v/$thickness_scint_strip; 
print "The V/W scintillators have: $number_of_strips_V_W strips \n\n";

my $fake_right_triangle_height = $thickness_scint_strip*19.5; #Read 'PCAL' documentation for why 19.5 is used
my $fake_opp = $fake_right_triangle_height;
my $new_gamma_hyp = $fake_opp/sin($gamma_angle_rad);
my $new_gamma_adj = cos($gamma_angle_rad/2)*$new_gamma_hyp;
my $y_shift = $new_gamma_adj;
my $y_correction_factor = 0.109; # millimeters

my $shifting_angle_rad = atan( (1/tan($gamma_angle_rad)) - (1/tan($symmetric_angle_rad)) );
my $shifting_angle_deg = $shifting_angle_rad*180/$pi;
print "The shifting angle (aka 'Alp angle') for V/W scintillator mother volume and strips is: $shifting_angle_deg degrees \n\n";

sub make_mother_scintillator_V 
{
	for(my $k = 2.0; $k <= $nlayers; $k+=3)
	{
		my $pDx1mom   = $width_pcal/2;
		my $pDx3mom   = $width_pcal/2;
		my $pDx2mom   = 0.00000001;
		my $pDx4mom   = 0.00000001;
		my $pDy1mom   = $height_v/2; 
		my $pDy2mom   = $height_v/2;
		my $pDzmom    = $dscint/2;
		my $pThetamom = 0;
		my $pPhimom   = $pThetamom;
		my $pAlp1mom  = -1*$shifting_angle_deg;   
		my $pAlp2mom  = $pAlp1mom;

		my $x_scint = $x_length_pcal/8;
		my $y_scint = $y_shift + $y_correction_factor; #This translates into: 963.287241326841 mm + .109 mm;
		my $z_scint = -$total_thickness/2 + ($k - 1)*($dscint + $dlead) + $dscint/2; 

		$detector{"name"}        = "V-view-scintillator ${k}";
		$detector{"mother"}      = "PCAL";
		$detector{"description"} = "Forward Calorimeter";
		$detector{"pos"}         = "${x_scint}*mm ${y_scint}*mm ${z_scint}*mm";
		$detector{"rotation"}    = "0*deg 0*deg ${alpha_angle_deg}*deg";
		$detector{"color"}       = "33ffcc";
		$detector{"type"}        = "G4Trap";
		$detector{"dimensions"}  = "${pDzmom}*mm ${pThetamom}*deg ${pPhimom}*deg ${pDy1mom}*mm ${pDx1mom}*mm ${pDx2mom}*mm ${pAlp1mom}*deg ${ pDy2mom}*mm ${pDx3mom}*mm ${pDx4mom}*mm ${pAlp2mom}*deg";
		$detector{"material"}    = "Air";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = $k;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 0;
		$detector{"sensitivity"} = "no"; 
		$detector{"hit_type"}    = "";   
		$detector{"identifiers"} = ""; 
	
		print_det(\%detector, $file);
	}
}

#V-view Scintillator Strips
sub make_scintillator_V 
{
	my $view = 2;  #All V scintillator layers will be assigned as 2
	my $strip_no = 0;  

	for (my $k = 2.0; $k <= $nlayers; $k+=3)
	{
		for(my $j = 0; $j < 78; $j++)
		{	
			$strip_no++;  
			
			my $x_scint = -1*$j*$thickness_scint_strip*tan($shifting_angle_rad) + ($height_v*tan($shifting_angle_rad))/2 - ($thickness_scint_strip*tan($shifting_angle_rad))/2; 
		   	my $y_scint = (-($height_v/2.0) + ($thickness_scint_strip/2.0) + ($j*$thickness_scint_strip) ); 
			my $z_scint = 0;
			
			my $pDx1 = ($width_pcal - $j*($diff_btw_pDx1_and_pDx2))/2 ; 
    			my $pDx3 = ($width_pcal - $j*($diff_btw_pDx1_and_pDx2))/2 ; 
			my $pDx2 = ($width_pcal - ($j + 1)*($diff_btw_pDx1_and_pDx2))/2 ;
    			my $pDx4 = ($width_pcal - ($j + 1)*($diff_btw_pDx1_and_pDx2))/2 ;	
			my $pDy1 = ($thickness_scint_strip)/2;
    			my $pDy2 = ($thickness_scint_strip)/2;
			my $pDz = $dscint/2;
    			my $pTheta = 0;
    			my $pPhi   = 0;
    			my $pAlp1 = -1*$shifting_angle_deg;  
    			my $pAlp2  = $pAlp1; 

			$detector{"name"}        = "V-view_strip ${strip_no}"; 
 			$detector{"mother"}      = "V-view-scintillator ${k}";
 			$detector{"description"} = "Forward Calorimeter scintillator layer";
 			$detector{"pos"}         = "${x_scint}*mm ${y_scint}*mm ${z_scint}*mm";
 			$detector{"rotation"}    = "0*deg 0*deg 0*deg";  
 			$detector{"color"}       = "6600ff";
 			$detector{"type"}        = "G4Trap";
			$detector{"dimensions"}  = "${pDz}*mm ${pTheta}*deg ${pPhi}*deg ${pDy1}*mm ${pDx1}*mm ${pDx2}*mm ${pAlp1}*deg ${pDy2}*mm ${pDx3}*mm ${pDx4}*mm ${pAlp2}*deg";
			$detector{"material"}    = "Scintillator";
 			$detector{"mfield"}      = "no";
 			$detector{"ncopy"}       = $strip_no; 
 			$detector{"pMany"}       = 1;
 			$detector{"exist"}       = 1;
 			$detector{"visible"}     = 1;
 			$detector{"style"}       = 0;
 			$detector{"sensitivity"} = "PCAL";
 			$detector{"hit_type"}    = "PCAL";
 			$detector{"identifiers"} = "sector ncopy 0 stack manual $k view manual $view strip manual $strip_no";

 			print_det(\%detector, $file);
		}
	}
}

sub make_mother_scintillator_W 
{
	for(my $k = 3.0; $k <= $nlayers; $k+=3)
	{
		my $pDx1mom   = $width_pcal/2;
		my $pDx3mom   = $width_pcal/2;
		my $pDx2mom   = 0.00000001;
		my $pDx4mom   = 0.00000001;
		my $pDy1mom   = $height_v/2; 
		my $pDy2mom   = $height_v/2;
		my $pDzmom    = $dscint/2;
		my $pThetamom = 0;
		my $pPhimom   = $pThetamom;
		my $pAlp1mom  = -1*$shifting_angle_deg;   
		my $pAlp2mom  = $pAlp1mom;

		my $x_scint = -$x_length_pcal/8;
		my $y_scint = $y_shift + $y_correction_factor; #This translates into: 963.287241326841 mm + .109 mm;
		my $z_scint = -$total_thickness/2 + ($k - 1)*($dscint + $dlead) + $dscint/2; 

		$detector{"name"}        = "W-view-scintillator ${k}";
		$detector{"mother"}      = "PCAL";
		$detector{"description"} = "Forward Calorimeter";
		$detector{"pos"}         = "${x_scint}*mm ${y_scint}*mm ${z_scint}*mm";
		$detector{"rotation"}    = "0*deg 180*deg ${alpha_angle_deg}*deg";
		$detector{"color"}       = "33ffcc";
		$detector{"type"}        = "G4Trap";
		$detector{"dimensions"}  = "${pDzmom}*mm ${pThetamom}*deg ${pPhimom}*deg ${pDy1mom}*mm ${pDx1mom}*mm ${pDx2mom}*mm ${pAlp1mom}*deg ${ pDy2mom}*mm ${pDx3mom}*mm ${pDx4mom}*mm ${pAlp2mom}*deg";
		$detector{"material"}    = "Air";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = $k;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 0;
		$detector{"sensitivity"} = "no";
		$detector{"hit_type"}    = "";
		$detector{"identifiers"} = "";
	
		print_det(\%detector, $file);
	}
}

#W-view Scintillator Strips
sub make_scintillator_W 
{
	my $view = 3;  #All W scintillator layers will be assigned as 3
	my $strip_no = 0;  

	for (my $k = 3.0; $k <= $nlayers; $k+=3)
	{
		for(my $j = 0; $j < 78; $j++) 
		{	
			$strip_no++;  			
			 
			my $x_scint = -1*$j*$thickness_scint_strip*tan($shifting_angle_rad) + ($height_v*tan($shifting_angle_rad))/2 - ($thickness_scint_strip*tan($shifting_angle_rad))/2; 
		   	my $y_scint = (-($height_v/2.0) + ($thickness_scint_strip/2.0) + ($j*$thickness_scint_strip) ); 
			my $z_scint = 0;
			
			my $pDx1 = ($width_pcal - $j*($diff_btw_pDx1_and_pDx2))/2 ; 
    			my $pDx3 = ($width_pcal - $j*($diff_btw_pDx1_and_pDx2))/2 ; 
			my $pDx2 = ($width_pcal - ($j + 1)*($diff_btw_pDx1_and_pDx2))/2 ;
    			my $pDx4 = ($width_pcal - ($j + 1)*($diff_btw_pDx1_and_pDx2))/2 ;	
			my $pDy1 = ($thickness_scint_strip)/2;
    			my $pDy2 = ($thickness_scint_strip)/2;
			my $pDz = $dscint/2;
    			my $pTheta = 0;
    			my $pPhi   = 0;
    			my $pAlp1 = -1*$shifting_angle_deg;  
    			my $pAlp2  = $pAlp1; 

			$detector{"name"}        = "W-view_strip ${strip_no}"; 
 			$detector{"mother"}      = "W-view-scintillator ${k}";
 			$detector{"description"} = "Forward Calorimeter scintillator layer";
 			$detector{"pos"}         = "${x_scint}*mm ${y_scint}*mm ${z_scint}*mm";
 			$detector{"rotation"}    = "0*deg 0*deg 0*deg";  
 			$detector{"color"}       = "cc66ff";
 			$detector{"type"}        = "G4Trap";
			$detector{"dimensions"}  = "${pDz}*mm ${pTheta}*deg ${pPhi}*deg ${pDy1}*mm ${pDx1}*mm ${pDx2}*mm ${pAlp1}*deg ${pDy2}*mm ${pDx3}*mm ${pDx4}*mm ${pAlp2}*deg";
			$detector{"material"}    = "Scintillator";
 			$detector{"mfield"}      = "no";
 			$detector{"ncopy"}       = $strip_no; 
 			$detector{"pMany"}       = 1;
 			$detector{"exist"}       = 1;
 			$detector{"visible"}     = 1;
 			$detector{"style"}       = 0;
 			$detector{"sensitivity"} = "PCAL";
 			$detector{"hit_type"}    = "PCAL";
 			$detector{"identifiers"} = "sector ncopy 0 stack manual $k view manual $view strip manual $strip_no";

 			print_det(\%detector, $file);
		}
	}
}

#SUB-ROUTINES

sub yplus 
{
    # calculates the top of the triangular active area of the PCAL (at large scattering angle)
    my $layer = $_[0];
    my $yplus = 1899.56 + 4.5419*($layer - 1);
    return $yplus;
}

sub yminus 
{
    # calculates the beam-side vertex of the triangular active area of the PCAL (at small scattering angle)
    my $layer = $_[0];
    my $yminus = -1829.74 - 4.3708*($layer - 1);
    return $yminus;
}

#BUILDS THE GEOMETRY

make_mother_PCAL();
make_lead();
make_scintillator_U();
make_mother_scintillator_V();
make_scintillator_V();
make_mother_scintillator_W();
make_scintillator_W();

