// %%%%%%%%%%%%%
// gemc headers
// %%%%%%%%%%%%%
#include "bmt_strip.h"

#include <iostream>
#include <cmath>
#include <cstdlib>

void bmt_strip::fill_infos()
{
 // all dimensions are in mm

 Pi  = 3.14159265358;
 interlayer 	= 10.0;            // distance between 2 layers of a superlayer

 
 pitch      	= 0.300;           // pitch of the strips
 Nsector    	= 3;

 // z of the upstream part of the layer
 Z0.push_back(-137.790);  Z0.push_back(-137.790);
 Z0.push_back(-155.880); Z0.push_back(-155.880);
 Z0.push_back(-172.530); Z0.push_back(-172.530);
 // Z0.push_back(-107.070); Z0.push_back(-107.070);

// total z length of the layer of the layer
 DZ.push_back(303.9);  DZ.push_back(303.9);
 DZ.push_back(397.0); DZ.push_back(397.0);
 DZ.push_back(430.3); DZ.push_back(430.3);

 // radii of layers
 R.push_back(146.700); R.push_back(R[0]+interlayer);
 R.push_back(181.700); R.push_back(R[2]+interlayer);
 R.push_back(216.700); R.push_back(R[4]+interlayer);
 // R.push_back(320.268/2); R.push_back(R[6]+interlayer);

 // Number of strips (depends on radius, dead zones, and pitch!)
 Nstrips.push_back(1020); Nstrips.push_back(1050);
 Nstrips.push_back(1260); Nstrips.push_back(1330);
 Nstrips.push_back(1510); Nstrips.push_back(1450);

 // mid angle of the sector
 MidTile.push_back(0);     MidTile.push_back(0);
 MidTile.push_back(0);     MidTile.push_back(0);
 MidTile.push_back(0); MidTile.push_back(0);
 // MidTile.push_back(0);     MidTile.push_back(0);

 DZ_inLength = 10.0;   // size of the band of dead zones all around in the length of the card
 DZ_inWidth  = 10.0;   // size of the band of dead zones all around in the width of the card
}

int bmt_strip::FindStrip(int layer, int sector, double x, double y, double z)
{
 const int MAXNSTRIP = 4096;

 if(Nstrips[layer] > MAXNSTRIP)
 {
   cout << "layer " << layer << ": Warning: number of strips > 4096. Exiting. " << Nstrips[layer] << endl;
   exit(0);
 }

  // 1st define phi of the hit point
  double phi;
  if(x>0 && y>=0) phi = atan(y/x);
  else if(x>0 && y<0) phi = 2.*Pi+atan(y/x);
  else if(x<0) phi = Pi+atan(y/x);
  else if(x==0 && y>0) phi = Pi/2.;
  else if(x==0 && y<0) phi = 3.*Pi/2.;
  else phi = 0; // x = y = 0, phi not defined

  // now find the tile number
  int ti=0;
  double theta_tmp=0;
  for(int t=0; t<Nsector; t++) 
  {
     theta_tmp = MidTile[layer]+2.*t*Pi/Nsector;
     if(theta_tmp>2.*Pi) theta_tmp = theta_tmp - 2.*Pi;
     if(theta_tmp<0) theta_tmp = theta_tmp + 2.*Pi;
     if(fabs(phi-theta_tmp)<Pi/Nsector || fabs(2.*Pi-fabs(phi-theta_tmp))<Pi/Nsector) ti=t; // gives tile #
  }

  double phiij = MidTile[layer]+2.*ti*Pi/Nsector;
  if(phi-phiij<-Pi/Nsector) phi = phi+2.*Pi;
  if(phi-phiij>Pi/Nsector) phi = phi-2.*Pi;
  if(phi-phiij<-Pi/Nsector || phi-phiij>Pi/Nsector) cout << "WARNING: incorrect phi value in BMT: " << phi*180./Pi << " vs " << phiij*180./Pi << endl;
  int ClosestStrip=0;
  if(layer%2==0) ClosestStrip = (int) (floor(((R[layer]/pitch)*(phi-phiij+Pi/Nsector-DZ_inLength/R[layer]))+0.5));
  if(layer%2==1) ClosestStrip = (int) (floor(((z-Z0[layer]-DZ_inWidth)/pitch))+0.5);
  
  int IsOK=1;
  // now check if the position is in the acceptance of the detector:
  if(ClosestStrip<0 || ClosestStrip>Nstrips[layer]) IsOK = 0;
 // particle is in the z-acceptance
  if((z<Z0[layer]+DZ_inWidth) || (z>Z0[layer]+DZ[layer]-DZ_inWidth)) IsOK=0;
  if((phi<phiij-Pi/Nsector+DZ_inLength/R[layer]) || (phi>phiij+Pi/Nsector-DZ_inLength/R[layer])) IsOK=0;

 if(IsOK) return ClosestStrip;
 else return -1;
 
}

