// %%%%%%%%%%%%%
// gemc headers
// %%%%%%%%%%%%%
#include "ftm_strip.h"

#include <iostream>
#include <cmath>

void ftm_strip::fill_infos()
{
 // all dimensions are in mm

 Pi  = 3.14159265358;
 interlayer      = (0.5+0.1+0.015+0.128+0.030+2.500)*2.; // distance between 2 layers of a superlayer
 intersuperlayer = 20.0;      // distance between 2 superlayers

 pitch      = 0.500;                         // pitch of the strips
 
 Rmin = 65.0;                 // inner radius of disks
 Rmax = 160.0;  // outer radius of disks
 Z_1stlayer = 1763.-0.5-0.1-0.015-0.128-0.030-2.500; // z position of the 1st layer

 // z of the upstream part of the layer
 Z0.push_back(Z_1stlayer);
 Z0.push_back(Z0[0]+interlayer);
 Z0.push_back(Z_1stlayer+intersuperlayer);
 Z0.push_back(Z0[2]+interlayer);

 // mid angle of the sector
 MidTile.push_back((179.0/2.0)*Pi/180.0);
 MidTile.push_back((179.5)*Pi/180.0);
 MidTile.push_back((179.0/2.0)*Pi/180.0);
 MidTile.push_back((179.5)*Pi/180.0);

 // Number of strips
 Nstrips = (int) floor((2.*Rmax)/pitch);

}

int ftm_strip::FindStrip(int layer, double x, double y, double z)
{

int ClosestStrip=0;
  if(layer%2==0) ClosestStrip = (int) (floor((x+160+0.5)/pitch)+1);
  if(layer%2==1) ClosestStrip = (int) (floor((y+160+0.5)/pitch)+1);

  int IsOK=1;

  // now check if the position is in the acceptance of the detector:
  if(ClosestStrip<0 || ClosestStrip>Nstrips) IsOK = 0;
  //if(layer%2==0 && 321-6<ClosestStrip && ClosestStrip<321 && y<0) IsOK = 0; // connection of the two parts of the detector
  //if(layer%2==0 && 321<ClosestStrip && ClosestStrip<321+6 && y>0) IsOK = 0;
  //if(layer%2==1 && 321-6<ClosestStrip && ClosestStrip<321 && x<0) IsOK = 0;
  //if(layer%2==1 && 321<ClosestStrip && ClosestStrip<321+6 && x>0) IsOK = 0;

  if(sqrt(x*x+y*y)< Rmin || sqrt(x*x+y*y)>Rmax)
  {
     IsOK = 0;
  }
  if(IsOK) return ClosestStrip;
  else return -1;

}

