// %%%%%%%%%%
// G4 headers
// %%%%%%%%%%
#include "G4Material.hh"
#include "G4Element.hh"
#include "G4NistManager.hh"
#include "G4OpBoundaryProcess.hh"

// %%%%%%%%%%%
// C++ headers
// %%%%%%%%%%%
#include <string>
#include <map>
using namespace std;

map<string, G4Material*> DefineMaterials()
{
	map<string, G4Material*> MMats;
	
	G4double a, z, density;
	G4int nel;
	
	G4Element* As = new G4Element("Arsen"   ,  "As", z=33, a=   74.92*g/mole);
	G4Element* Ar = new G4Element("Argon",     "Ar", z=18, a=   39.95*g/mole);
	G4Element* Al = new G4Element("Aluminum",  "Al", z=13, a=  26.982*g/mole);
	//	G4Element* Au = new G4Element("Au",        "Au", z=79, a= 196.967*g/mole);
	G4Element* Ba = new G4Element("Barium",    "Ba", z=56, a=  137.327*g/mole);	
	G4Element* C  = new G4Element("Carbon",    "C",  z=6,  a=   12.01*g/mole);
	G4Element* Ca  = new G4Element("Calcium", "Ca",  z=20, a=   40.078*g/mole);	
	G4Element* Cr = new G4Element("Chromium",  "Cr", z=24, a=   52.00*g/mole);
	G4Element* Cl = new G4Element("Chlorine",  "Cl", z=17, a=   35.453*g/mole);
	//	G4Element* D  = new G4Element("Deuterium", "D",  z=1,  a=    2.01*g/mole);
	G4Element* F  = new G4Element("Fluorine",  "F",  z=9,  a= 18.9984*g/mole);
	G4Element* Fe = new G4Element("Iron",      "Fe", z=26, a=  55.845*g/mole);
	G4Element* H  = new G4Element("Hydrogen",  "H",  z=1,  a=    1.01*g/mole);
	G4Element* Mn = new G4Element("Manganese", "Mn", z=25, a=   54.94*g/mole);
	G4Element* He = new G4Element("Helium",    "He", z=2,  a=  4.0026*g/mole);
	G4Element* He3 = new G4Element("Helium3",  "He3", z=2, a=  3.0160293*g/mole);
	G4Element* N  = new G4Element("Nitrogen",  "N",  z=7,  a=   14.01*g/mole);
	//	G4Element* Ne = new G4Element("Neon",      "Ne", z=10, a=   20.1797*g/mole);
	G4Element* Ni = new G4Element("Nickel",    "Ni", z=28, a=   58.70*g/mole);
	G4Element* O  = new G4Element("Oxygen",    "O",  z=8,  a=   16.00*g/mole);
	G4Element* Pb = new G4Element("Lead",      "Pb", z=82, a=  207.19*g/mole);
	G4Element* Si = new G4Element("Silicon",   "Si", z=14, a=   28.09*g/mole);
	G4Element* Sr = new G4Element("Strontium", "Sr", z=38, a=   87.62*g/mole);	
	G4Element* W  = new G4Element("Tungsten",  "W",  z=74, a=  183.85*g/mole);
	G4Element* Wf = new G4Element("Wf",        "Wf", z=74, a=  183.85*g/mole);
	G4Element* Ox = new G4Element("Ox",        "Ox", z=8,  a=    16.0*g/mole);

	
	G4Material *Kryptonite = new G4Material("Kryptonite", density= 0.00000001*mg/cm3, nel=1);
	Kryptonite->AddElement(Ar, 100.*perCent);
	
	G4Material *Air = new G4Material("Air", density= 1.29*mg/cm3, nel=2);
	Air->AddElement(N, 70.*perCent);
	Air->AddElement(O, 30.*perCent);
	
	G4Material *Air_Opt = new G4Material("Air_Opt",   density= 1.29*mg/cm3, nel=2);
	Air_Opt->AddElement(N, 70.*perCent);
	Air_Opt->AddElement(O, 30.*perCent);
	
	G4Material *Air_Opt2 = new G4Material("Air_Opt2", density= 1.29*mg/cm3, nel=2);
	Air_Opt2->AddElement(N, 70.*perCent);
	Air_Opt2->AddElement(O, 30.*perCent);
	
	G4Material *Aerogel = new G4Material("Aerogel",   density= 0.14*g/cm3, nel=2);
	Aerogel->AddElement(Si, 30.*perCent);
	Aerogel->AddElement(O,  70.*perCent);
	
	G4Material *CCGas = new G4Material("CCGas",       density= 0.01012*g/cm3, nel=2);
	CCGas->AddElement(F, 70.*perCent);
	CCGas->AddElement(C, 30.*perCent);
	
	
	G4Material *Carbon       = new G4Material("Carbon",       z=6,  a=  12.011*g/mole,  density =  2.265*g/cm3);
	G4Material *CarbonFiber  = new G4Material("CarbonFiber",  z=6,  a=  12.011*g/mole,  density =  1.750*g/cm3);
	G4Material *LD2          = new G4Material("LD2",          z=1,  a=    2.01*g/mole,  density =  0.169*g/cm3);
	G4Material *LHe          = new G4Material("LHe",          z=2,  a=  4.0026*g/mole,  density =  0.125*g/cm3);
	
	// aluminum honeycomb core  (it is actually made of alloy Alu-Alloy 3003 (AlMnCu)
	G4Material *AlHoneycomb = new G4Material("AlHoneycomb", z=13, a=  26.982*g/mole, density =  0.13*g/cm3);
  
	// a denser aluminum honeycomb core for comparison
	G4Material *AlHoneycomb2 = new G4Material("AlHoneycomb2", z=13, a=  26.982*g/mole, density =  1.00*g/cm3);

	// Densimet-180 alloy (roughly)
	G4Material *Densimet = new G4Material("Densimet",   density = 18.0*g/cm3, nel=3);
	Densimet->AddElement(W, 90*perCent);
	Densimet->AddElement(Ni, 5*perCent);
	Densimet->AddElement(Fe, 5*perCent);
  

  
  
	G4Material *LeadTungsten = new G4Material("LeadTungsten",   density = 8.28*g/cm3, nel=3);
	LeadTungsten->AddElement(Pb,1);
	LeadTungsten->AddElement(W, 1);
	LeadTungsten->AddElement(O, 4);
	
	G4NistManager* matman = G4NistManager::Instance();
	
	G4Material* Copper   = matman->FindOrBuildMaterial("G4_Cu");       // Copper
	G4Material* LeadOxide   = matman->FindOrBuildMaterial("G4_LEAD_OXIDE");       // PbO
	G4Material* SilicOxide  = matman->FindOrBuildMaterial("G4_SILICON_DIOXIDE");  // SiO2	2.32g/cm3
	G4Material* PotasOxide  = matman->FindOrBuildMaterial("G4_POTASSIUM_OXIDE");  // K2O
	G4Material* SodMonOxide = matman->FindOrBuildMaterial("G4_SODIUM_MONOXIDE");  // Na2O
	
	G4Material* As2O3 = new G4Material("As2O3", density= 3.738*g/cm3, nel=2); // As2O3
	As2O3->AddElement( As, 2 );
	As2O3->AddElement( O , 3 );
	
	G4Material* LgTF1 = new G4Material("LgTF1", density= 3.86*g/cm3 , nel=5 );   // Lead-Glass
	LgTF1->AddMaterial( LeadOxide   , 0.5120 );
	LgTF1->AddMaterial( SilicOxide  , 0.4130 );
	LgTF1->AddMaterial( PotasOxide  , 0.0422 );
	LgTF1->AddMaterial( SodMonOxide , 0.0278 );
	LgTF1->AddMaterial( As2O3       , 0.0050 );
	
	G4Material *Scintillator = new G4Material("Scintillator",   density = 1.032*g/cm3, nel=2);
	Scintillator->AddElement(C, 9);
	Scintillator->AddElement(H, 10);
	
	G4Material *ScintillatorB = new G4Material("ScintillatorB",   density = 1.032*g/cm3, nel=2);
	ScintillatorB->AddElement(C, 9);
	ScintillatorB->AddElement(H, 10);
	ScintillatorB->GetIonisation()->SetBirksConstant(0.126*mm/MeV);
	
	G4Material *StainlessSteel = new G4Material("StainlessSteel",   density = 8.02*g/cm3, nel=5);
	StainlessSteel->AddElement(Mn, 0.02);
	StainlessSteel->AddElement(Si, 0.01);
	StainlessSteel->AddElement(Cr, 0.19);
	StainlessSteel->AddElement(Ni, 0.10);
	StainlessSteel->AddElement(Fe, 0.68);
	
	G4Material *Kapton = new G4Material("Kapton",   density = 1.42*g/cm3, nel=4);
	Kapton->AddElement(H, 0.026362);
	Kapton->AddElement(C, 0.691133);
	Kapton->AddElement(N, 0.073270);
	Kapton->AddElement(O, 0.209235);
	
	G4Material *Peek = new G4Material("Peek",   density = 1.31*g/cm3, nel=3);
	Peek->AddElement(H, 18);
	Peek->AddElement(C, 12);
	Peek->AddElement(O, 3);
	
	G4Material *OptScint     = new G4Material("OptScint",       density = 1.032*g/cm3, nel=2);
	OptScint->AddElement(C, 9);
	OptScint->AddElement(H, 10);
	
	G4Material *DCgas    = new G4Material("DCgas",              density = 1.8*mg/cm3, nel=3);
	DCgas->AddElement(Ar, 90*perCent);
	DCgas->AddElement(O,  6.6*perCent);
	DCgas->AddElement(C,  3.4*perCent);
	
	G4Material *Epoxy    = new G4Material("Epoxy",              density = 1.782*mg/cm3, nel=3);
	Epoxy->AddElement(C, 80.26*perCent);
	Epoxy->AddElement(H, 7.86*perCent);
	Epoxy->AddElement(O, 11.88*perCent);
	
	G4Material *Rohacell    = new G4Material("Rohacell",        density = 100*kg/m3, nel=4);
	Rohacell->AddElement(C, 64.65*perCent);
	Rohacell->AddElement(H, 7.84*perCent);
	Rohacell->AddElement(N, 8.39*perCent);
	Rohacell->AddElement(O, 19.14*perCent);
	
	G4Material *FTinsfoam    = new G4Material("FTinsfoam",     density = 34*kg/m3, nel=4);
	FTinsfoam->AddElement(C, 60.*perCent);
	FTinsfoam->AddElement(H, 10.*perCent);
	FTinsfoam->AddElement(N, 10.*perCent);
	FTinsfoam->AddElement(O, 20.*perCent);
	
	G4Material *Noryl	= new G4Material("Noryl",	    density = 1.06*g/cm3, nel=3);
	Noryl->AddElement(C, 47.06*perCent );
	Noryl->AddElement(H, 47.06*perCent );
	Noryl->AddElement(O,  5.88*perCent );
	
	G4Material *PCBoardM    = new G4Material("PCBoardM",        density = 1.860*g/cm3, nel=3);
	PCBoardM->AddElement(Fe,  30*perCent);
	PCBoardM->AddElement(C,   40*perCent);
	PCBoardM->AddElement(Si,  30*perCent);
	
	G4Material *svtwirebond    = new G4Material("svtwirebond",  density = 2.69*g/cm3, nel=2);
	svtwirebond->AddElement(Al, 99*perCent);
	svtwirebond->AddElement(Si, 1*perCent);
	
	G4Material *MMGas = new G4Material("MMGas",   density = (1.662*0.95+2.489*0.05)*mg/cm3, nel=3);
	MMGas->AddElement(Ar, 95.0*perCent);
	MMGas->AddElement(H, 0.173414*5.0*perCent);
	MMGas->AddElement(C, 0.826586*5.0*perCent);
	
	
	 	// BST Support Material: Copper + HFCB
  // All dimensions in mm
	//          Size                 %
	// Gold:    0.0600              1.6
	// Copper:  0.1397 + 2.5       68.9
	// Polymide 0.2154              5.6    (Kapton)
	// FR4      0.9144             23.9 
	// Total:   1.3295 + 2.5 = 3.8925
//	G4Material *BSTSupport = new G4Material("BSTSupport",   density = 9.0*g/cm3, nel=3);
//	BSTSupport->AddElement(G4_Au,        1.6*perCent);
//	BSTSupport->AddElement(G4_Cu,       68.9*perCent);
//	BSTSupport->AddElement(Kapton,       5.6*perCent);
//	BSTSupport->AddElement(CarbonFiber, 23.9*perCent);
	
		
	/*
	G4Material *MMGas = new G4Material("MMGas",   density = 1.17*mg/cm3, nel=4);
	MMGas->AddElement(Ne, 79.0*perCent);
	MMGas->AddElement(H, 0.2011*11.0*perCent);
	MMGas->AddElement(C, (0.7989*11.0+0.1365*10.0)*perCent);
	MMGas->AddElement(F, 0.8635*10.0*perCent);
	*/
	/*
	G4Material *MMGas = new G4Material("MMGas",   density = 1.87*mg/cm3, nel=4);
	MMGas->AddElement(Ar, 95.0*perCent);
	MMGas->AddElement(H, 0.1734*2.0*perCent);
	MMGas->AddElement(C, (0.8266*2.0+0.1365*3.0)*perCent);
	MMGas->AddElement(F, 0.8635*3.0*perCent);
	*/
	
	G4double MMStripTransparency = 300./400.;
	G4Material *MMStrips = new G4Material("Copper", z=29, a=   63.55*g/mole, density = 8.960*MMStripTransparency*g/cm3);
	
	//G4double MMMeshTransparency = (19./50.)*(19./50.);
	G4double MMMeshTransparency = 1.0;
	
	G4Material *MMMesh = new G4Material("MMMesh",   density = 8.02*MMMeshTransparency*g/cm3, nel=5);
	MMMesh->AddElement(Mn, 0.02);
	MMMesh->AddElement(Si, 0.01);
	MMMesh->AddElement(Cr, 0.19);
	MMMesh->AddElement(Ni, 0.10);
	MMMesh->AddElement(Fe, 0.68);

	/*
	G4Material *MMMesh       = new G4Material("MMMesh",  z=28, a=   58.70*g/mole,  density =  8.902*MMMeshTransparency*g/cm3);
	*/
	
	G4Material *MMMylar = new G4Material("MMMylar",   density = 1.40*g/cm3, nel=3);
	MMMylar->AddElement(H, 0.041958);
	MMMylar->AddElement(C, 0.625017);
	MMMylar->AddElement(O, 0.333025);
	
	/*
	G4Material *MMMylar = new G4Material("MMMylar",   density = 8.02*MMMeshTransparency*g/cm3, nel=5);
	MMMylar->AddElement(Mn, 0.02);
	MMMylar->AddElement(Si, 0.01);
	MMMylar->AddElement(Cr, 0.19);
	MMMylar->AddElement(Ni, 0.10);
	MMMylar->AddElement(Fe, 0.68);
	*/
	//G4Material *MMMylar       = new G4Material("MMMylar",  z=28, a=   58.70*g/mole,  density =  8.902*g/cm3);
	G4Material* He4_1atm = new G4Material( "He4_1atm",  density = 1.*0.1786*mg/cm3, nel=1 );
	He4_1atm->AddElement( He, 100.0*perCent );
	G4Material* He4_2atm = new G4Material( "He4_2atm",  density = 2.*0.1786*mg/cm3, nel=1 );
	He4_2atm->AddElement( He, 100.0*perCent );
	G4Material* He4_3atm = new G4Material( "He4_3atm",  density = 3.*0.1786*mg/cm3, nel=1 );
	He4_3atm->AddElement( He, 100.0*perCent );
	G4Material* He4_7atm = new G4Material( "He4_7atm",  density = 7.*0.1786*mg/cm3, nel=1 );
	He4_7atm->AddElement( He, 100.0*perCent );
	
	G4Material* PbWO4    = new G4Material( "PbWO4",  density = 8.28*g/cm3, nel=3 );
	PbWO4->AddElement( Pb,   1./6.*100.*perCent );
	PbWO4->AddElement( Wf,   1./6.*100.*perCent );
	PbWO4->AddElement( Ox,   4./6.*100.*perCent );
	
  G4Material* SemiMirror    = new G4Material( "SemiMirror",  density = 8.28*g/cm3, nel=3 );
	SemiMirror->AddElement( Pb,   1./6.*100.*perCent );
	SemiMirror->AddElement( Wf,   1./6.*100.*perCent );
	SemiMirror->AddElement( Ox,   4./6.*100.*perCent );

	//for pol He3 target
	//He3 gas target
	G4Material* He3_10amg = new G4Material( "He3_10amg",  density = 10.*0.1345*mg/cm3, nel=1 );
	//0.1345=44.6(amg=mol/m3)*3.016(g/mol)
	He3_10amg->AddElement( He3, 100.0*perCent );

	//He3 cell glass
	G4Material *BariumOxide = new G4Material("BariumOxide", density=5.72*g/cm3, nel=2);
	BariumOxide->AddElement(Ba, 1);
	BariumOxide->AddElement(O,  1);

	G4Material *StrontiumOxide = new G4Material("StrontiumOxide", density=4.7*g/cm3, nel=2);
	StrontiumOxide->AddElement(Sr, 1);
	StrontiumOxide->AddElement(O,  1);
	
	G4Material *AluminiumOxide =  matman->FindOrBuildMaterial("G4_ALUMINUM_OXIDE");  // Al2O3	3.97g/cm3
	G4Material *CalciumOxide =  matman->FindOrBuildMaterial("G4_CALCIUM_OXIDE");  // CaO	3.3g/cm3

	G4Material *Glass_GE180 = new G4Material("Glass_GE180", density=2.76*g/cm3, nel=5);
	Glass_GE180->AddMaterial(SilicOxide, 60.8*perCent);   //add 0.5 to make total as 1
	Glass_GE180->AddMaterial(BariumOxide, 18.2*perCent);
	Glass_GE180->AddMaterial(AluminiumOxide, 14.3*perCent);
	Glass_GE180->AddMaterial(CalciumOxide, 6.5*perCent);
	Glass_GE180->AddMaterial(StrontiumOxide, 0.25*perCent);
	//http://galileo.phys.virginia.edu/research/groups/spinphysics/glass_properties.html
	
// 	G4Material *Glass_GE180 = new G4Material("Glass_GE180", density=2.76*g/cm3, nel=6);
// 	Glass_GE180->AddElement(Si, 1./3.*60.3*perCent);
// 	Glass_GE180->AddElement(Ba, 1./2.*18.2*perCent);
// 	Glass_GE180->AddElement(Al, 2./5.*14.3*perCent);
// 	Glass_GE180->AddElement(Ca, 1./2.*6.5*perCent);
// 	Glass_GE180->AddElement(Sr, 1./2.*0.25*perCent);	
// 	Glass_GE180->AddElement(O, (2./3.*60.3+1./2.*18.2+3./5.*14.3+1./2.*6.5+1./2.*0.25)*perCent);		

	//proton pol target NH3
	//solid NH3
	G4double density_NH3_solid = 0.817*g/cm3;
	G4Material *NH3_solid = new G4Material("NH3_solid", density_NH3_solid, nel=2);
	NH3_solid->AddElement(H, 3);
	NH3_solid->AddElement(N, 1);
	
	G4Material *He4_liquid = new G4Material( "He4_liquid",  density = 0.145*g/cm3, nel=1 );
	He4_liquid->AddElement( He, 100.0*perCent );	

	//SolidNH3(55%)+LiquidHe(45%) in volumn?
	//density = mLiquidHeD*(1.0-mNH3VolumnRatio)+mSolidNH3D*mNH3VolumnRatio;
	//density_NH3He = 1.0/((1.0-mNH3WeightRatio)/mLiquidHeD+mNH3WeightRatio/mSolidNH3D);
	G4double density_NH3He = (0.817*0.55+0.145*0.45)*g/cm3;	
	G4Material *NH3He = new G4Material("NH3He", density_NH3He, nel=2);
	NH3He->AddMaterial(NH3_solid, 0.8732);
	NH3He->AddMaterial(He4_liquid, 1-0.8732);

	//Beryllium oxide
	G4Material *BerylliumOxide =  matman->FindOrBuildMaterial("G4_BERYLLIUM_OXIDE");  // BeO 3.02g/cm3
	
	// polyethylene
	G4Material *polyethylene =  matman->FindOrBuildMaterial("G4_POLYETHYLENE");  
	
	// for GEM
	// NOMEX:
	G4Material* NOMEX_pure = new G4Material("NOMEX_pure", density = 1.38*g/cm3, 5);
	NOMEX_pure -> AddElement(H,0.04);
	NOMEX_pure -> AddElement(C,0.54);
	NOMEX_pure -> AddElement(N,0.09);
	NOMEX_pure -> AddElement(O,0.10);
	NOMEX_pure -> AddElement(Cl,0.23);
  
	G4Material* NOMEX = new G4Material("NOMEX",density = 0.04*g/cm3, 2);
	NOMEX -> AddMaterial(NOMEX_pure,0.45);
	NOMEX -> AddMaterial(Air,0.55);
	
	// Nema G10:
  	G4Material* NEMAG10 = new G4Material("NEMAG10", 1.70*g/cm3, nel=4);
        NEMAG10 -> AddElement(Si, 1);
	NEMAG10 -> AddElement(O , 2);
	NEMAG10 -> AddElement(C , 3);
	NEMAG10 -> AddElement(H , 3);

	// Ar (70) CO2 (30) STP
	G4double density_Ar = 1.7823*mg/cm3 ;
	G4Material* Argon = new G4Material("Argon"  , density_Ar, nel=1);
	Argon->AddElement(Ar, 1);
  
	G4double density_CO2 = 1.977*mg/cm3;
	G4Material* CO2 = new G4Material("CO2", density_CO2, nel=2);
	CO2->AddElement(C,1);
	CO2->AddElement(O,2);	

	G4double density_ArCO2 = .7*density_Ar + .3*density_CO2;
	G4Material *ArCO2 = new G4Material("GEMgas", density_ArCO2, nel=2);
	ArCO2->AddMaterial(Argon, 0.7*density_Ar/density_ArCO2) ;
	ArCO2->AddMaterial(CO2, 0.3*density_CO2/density_ArCO2) ;
	
		
		//RICH MATERIAL
	G4Material *C6F14 = new G4Material("C6F14", density=1.680*g/cm3, nel=2);
	C6F14->AddElement(C,  6);
	C6F14->AddElement(F, 14);
	
	G4Material *C5F12 = new G4Material("C5F12", density=1.680*g/cm3, nel=2);
	C5F12->AddElement(C ,   5);
	C5F12->AddElement(F ,  12);
	
	G4Material* H2O = new G4Material("H20", density=1.000*g/cm3, nel=2);
	H2O->AddElement(H, 2);
	H2O->AddElement(O, 1);
	
	G4Material *AerogelQuartz = new G4Material("AerogelQuartz", density=2.200*g/cm3, nel=2);
	AerogelQuartz->AddElement(Si, 1);
	AerogelQuartz->AddElement(O,  2);
	
	G4Material *RichAerogel = new G4Material("RichAerogel", density=0.200*g/cm3, nel=2);
		//G4Material *RichAerogel = new G4Material("RichAerogel", density=0.100*g/cm3, nel=2);	
		//G4Material *RichAerogel = new G4Material("RichAerogel", density=0.000*g/cm3, nel=2);
	
	RichAerogel->AddMaterial(AerogelQuartz, 97.0*perCent);
	RichAerogel->AddMaterial(H2O,          3.0*perCent);
	
	G4Material *Quartz = new G4Material("Quartz", density= 4.400*g/cm3, nel=2);
	Quartz->AddElement(Si, 1);
	Quartz->AddElement(O,  2);
	
		//G4Material *Methane = new G4Material("Methane", density= 0.422*g/cm3, nel=2);  //liquid
		//G4Material *Methane = new G4Material("Methane", density= 0.717*kg/m3, nel=2);  //gas at 0 degree, 1atm
	G4Material *Methane = new G4Material("Methane", density= 0.667*kg/m3, nel=2);    //gas at 20 degree, 1atm
	Methane->AddElement(C, 1);
	Methane->AddElement(H, 4);
	
	G4Material *Alumi = new G4Material("Alumi", density=2.7*g/cm3, nel=1);
	Alumi->AddElement(Al, 1);
	
	G4Material *Glass = new G4Material("Glass", density=1.032*g/cm3, nel=2);
	Glass->AddElement(C, 91.533*perCent);
	Glass->AddElement(H,  8.467*perCent);
	
	G4Material *BusCable = new G4Material("BusCable", density=1.200*g/cm3, nel=2);
	BusCable->AddMaterial(Copper,        24.0*perCent);
	BusCable->AddMaterial(polyethylene,  76.0*perCent);
	
		//END OF RICH MATERIAL
	
	MMats["AerogelQuartz"]  = AerogelQuartz;
	MMats["Air"]            = Air;
	MMats["Air_Opt"]        = Air_Opt;
	MMats["Air_Opt2"]       = Air_Opt2;
	MMats["Alumi"]          = Alumi;
	MMats["Aluminum"]       = matman->FindOrBuildMaterial("G4_Al");
	MMats["Aerogel"]        = Aerogel;
	MMats["BusCable"]       = BusCable;
	MMats["C5F12"]          = C5F12;
	MMats["C6F14"]          = C6F14;
	MMats["Carbon"]         = Carbon;
	MMats["CarbonFiber"]    = CarbonFiber;
	MMats["CCGas"]          = CCGas;
	MMats["Concrete"]       = matman->FindOrBuildMaterial("G4_CONCRETE");       // concrete
	MMats["Copper"]         = matman->FindOrBuildMaterial("G4_Cu");
	MMats["DCgas"]          = DCgas;
	MMats["Epoxy"]          = Epoxy;
	MMats["FTinsfoam"]      = FTinsfoam;
	MMats["Glass"]          = Glass;
	MMats["Glass_GE180"]    = Glass_GE180;
	MMats["Gold"]           = matman->FindOrBuildMaterial("G4_Au");
	MMats["H2O"]            = H2O;
	MMats["He4_1atm"]       = He4_1atm ;
	MMats["He4_2atm"]       = He4_2atm ;
	MMats["He4_3atm"]       = He4_3atm ;
	MMats["He4_7atm"]       = He4_7atm ;
	MMats["He3_10amg"]      = He3_10amg ;
	MMats["Iron"]           = matman->FindOrBuildMaterial("G4_Fe");
	MMats["Kapton"]         = Kapton;
	MMats["Kryptonite"]     = matman->FindOrBuildMaterial("Kryptonite");
	MMats["LD2"]            = LD2;
	MMats["Lead"]           = matman->FindOrBuildMaterial("G4_Pb");
	MMats["LeadTungsten"]   = LeadTungsten;
	MMats["LgTF1"]          = LgTF1;                                     // Lead-Glass
	MMats["LH2"]            = matman->FindOrBuildMaterial("G4_lH2");;
	MMats["LHe"]            = LHe;
	MMats["Methane"]        = Methane;
	MMats["MMGas"]          = MMGas;
	MMats["MMStrips"]       = MMStrips;
	MMats["MMMesh"]         = MMMesh;
	MMats["MMMylar"]        = MMMylar;
	MMats["Nickel"]         =  matman->FindOrBuildMaterial("G4_Ni");
	MMats["Noryl"]          = Noryl ;
	MMats["Peek"]           = Peek ;
	MMats["PCBoardM"]       = PCBoardM ;
	MMats["PbWO4"]          = PbWO4 ;
	MMats["Quartz"]         = Quartz;
	MMats["OptScint"]       = OptScint;
	MMats["Rohacell"]       = Rohacell ;
	MMats["RichAerogel"]    = RichAerogel;
	MMats["Scintillator"]   = Scintillator;
	MMats["ScintillatorB"]  = ScintillatorB;
	MMats["SemiMirror"]     = SemiMirror ;
	MMats["Silicium"]       = matman->FindOrBuildMaterial("G4_Si");
	MMats["Silicon"]        = matman->FindOrBuildMaterial("G4_Si");
	MMats["StainlessSteel"] = StainlessSteel;
	MMats["svtwirebond"]    = svtwirebond;
	MMats["Teflon"]         = matman->FindOrBuildMaterial("G4_TEFLON");
	MMats["Tungsten"]       = matman->FindOrBuildMaterial("G4_W");
	MMats["Vacuum"]         = matman->FindOrBuildMaterial("G4_Galactic");
	MMats["Water"]          = matman->FindOrBuildMaterial("G4_WATER");
	MMats["AlHoneycomb"]    = AlHoneycomb;
	MMats["AlHoneycomb2"]   = AlHoneycomb2;
	MMats["Densimet"]       = Densimet;
	MMats["NOMEX_pure"]     = NOMEX_pure;
	MMats["NOMEX"]          = NOMEX;
	MMats["NEMAG10"]        = NEMAG10;
	MMats["Argon"]          = Argon;
	MMats["CO2"]            = CO2;
	MMats["ArCO2"]          = ArCO2;
	MMats["NH3_solid"]      = NH3_solid;
	MMats["He4_liquid"]     = He4_liquid;
	MMats["NH3He"]          = NH3He;
	MMats["BerylliumOxide"] = BerylliumOxide;
	
	
	// %%%%%%%%%%%%%%%%%%%%%%%%%%%%
	// Materials Optical Properties
	// %%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	// Optical Scintillator
	const G4int nEntries_OptScint = 2;
	
	G4double PhotonEnergy_OptScint[nEntries_OptScint]    = { 2.034*eV , 4.136*eV };
	G4double RefractiveIndex_OptScint[nEntries_OptScint] = { 1.49     , 1.49   };
	G4double Absorption_OptScint[nEntries_OptScint]      = { 4.5*m    , 4.5*m  };
	G4double ScintilFast_OptScint[nEntries_OptScint]     = { 1.00     , 1.00   };
	G4double ScintilSlow_OptScint[nEntries_OptScint]     = { 10.00    , 10.00  };
	
	G4MaterialPropertiesTable* OptScint_MPT = new G4MaterialPropertiesTable();
	OptScint_MPT->AddProperty("RINDEX",        PhotonEnergy_OptScint, RefractiveIndex_OptScint, nEntries_OptScint);
	OptScint_MPT->AddProperty("ABSLENGTH",     PhotonEnergy_OptScint, Absorption_OptScint,      nEntries_OptScint);
	OptScint_MPT->AddProperty("FASTCOMPONENT", PhotonEnergy_OptScint, ScintilFast_OptScint,     nEntries_OptScint);
	OptScint_MPT->AddProperty("SLOWCOMPONENT", PhotonEnergy_OptScint, ScintilSlow_OptScint,     nEntries_OptScint);
	
	OptScint_MPT->AddConstProperty("SCINTILLATIONYIELD", 1./MeV);
	OptScint_MPT->AddConstProperty("RESOLUTIONSCALE",    1.0);
	OptScint_MPT->AddConstProperty("FASTTIMECONSTANT",   2.1*ns);
	OptScint_MPT->AddConstProperty("SLOWTIMECONSTANT",   10.*ns);
	OptScint_MPT->AddConstProperty("YIELDRATIO",         0.1);
	MMats["OptScint"]->SetMaterialPropertiesTable(OptScint_MPT);
	
	// Lead Glass
	const G4int nEntries = 15;
	G4double PhotonEnergyTF1[nEntries] =
	{ 1.99988723*eV, 2.06655002*eV, 2.13781047*eV,
	2.21416092*eV, 2.29616666*eV, 2.38448095*eV,
	2.47986007*eV, 2.58318758*eV, 2.69550014*eV,
	2.81802297*eV, 2.95221448*eV, 3.09982514*eV,
	3.26297379*eV, 3.44425011*eV, 3.64685321*eV };
	
	G4double RefractiveIndex1[nEntries] =
	{ 1.6436, 1.6454, 1.6474, 1.6496, 1.6521,
	1.6548, 1.6579, 1.6614, 1.6653, 1.6697,
	1.6748, 1.6806, 1.6874, 1.6952, 1.7045 };
	
	G4double Absorption1[nEntries] =
	{ 7.0*m, 7.0*m, 7.0*m, 7.0 *m, 7.0 *m,
	  7.0*m, 6.0*m, 5.0*m, 3.5 *m, 2.75*m,
	  1.8*m, 1.0*m, 0.5*m, 0.15*m, 0.08*m };
	
	G4MaterialPropertiesTable* TF1glass = new G4MaterialPropertiesTable();
	TF1glass->AddProperty("RINDEX",       PhotonEnergyTF1, RefractiveIndex1, nEntries);
	TF1glass->AddProperty("ABSLENGTH",    PhotonEnergyTF1, Absorption1,      nEntries);	
	MMats["LgTF1"]->SetMaterialPropertiesTable(TF1glass);
	
	
	// Air Reflection
	const G4int nEntries_Air = 2;
	G4double PhotonEnergy_Air[nEntries_Air]    = { 2.034*eV , 4.136*eV };
	G4double RefractiveIndex_Air[nEntries_Air] = { 1.00, 1.00 };
	
	G4MaterialPropertiesTable* Air_MPT = new G4MaterialPropertiesTable();
	Air_MPT->AddProperty("RINDEX", PhotonEnergy_Air, RefractiveIndex_Air, nEntries_Air);
	MMats["Air_Opt"]->SetMaterialPropertiesTable(Air_MPT);
// 	MMats["Air"]->SetMaterialPropertiesTable(Air_MPT);
	
	
	// Air2 Reflection
	G4double RefractiveIndex_Air2[nEntries_Air] = { 1.00, 1.00 };
	G4double Absorption_Air2[nEntries_Air]      = { 0.0*mm    , 0.0*mm  };
	
	G4MaterialPropertiesTable* Air_MPT2 = new G4MaterialPropertiesTable();
	Air_MPT2->AddProperty("RINDEX",     PhotonEnergy_Air, RefractiveIndex_Air2, nEntries_Air);
	Air_MPT2->AddProperty("ABSLENGTH",  PhotonEnergy_Air, Absorption_Air2,      nEntries_Air);
	MMats["Air_Opt2"]->SetMaterialPropertiesTable(Air_MPT2);
	
  G4double PhotonEnergy_SemiMirror[nEntries_Air]    = { 2.034*eV , 4.136*eV };
	G4double RefractiveIndex_SemiMirror[nEntries_Air] = { 5.00, 5.00 };
  G4double Absorption_SemiMirror[nEntries_Air]      = { 100.0*m    , 100.0*m  };
  
	G4MaterialPropertiesTable* SemiMirrorMPT = new G4MaterialPropertiesTable();
	SemiMirrorMPT->AddProperty("RINDEX",     PhotonEnergy_SemiMirror, RefractiveIndex_SemiMirror, 2);
	SemiMirrorMPT->AddProperty("ABSLENGTH",  PhotonEnergy_SemiMirror, Absorption_SemiMirror,      2);
	MMats["SemiMirror"]->SetMaterialPropertiesTable(SemiMirrorMPT);
	

  
  
	// Water Properties
	const G4int nEntries_Water = 32;
	G4double PhotonEnergy_Water[nEntries_Water] =
	{ 2.034*eV, 2.068*eV, 2.103*eV, 2.139*eV,
		2.177*eV, 2.216*eV, 2.256*eV, 2.298*eV,
		2.341*eV, 2.386*eV, 2.433*eV, 2.481*eV,
		2.532*eV, 2.585*eV, 2.640*eV, 2.697*eV,
		2.757*eV, 2.820*eV, 2.885*eV, 2.954*eV,
		3.026*eV, 3.102*eV, 3.181*eV, 3.265*eV,
		3.353*eV, 3.446*eV, 3.545*eV, 3.649*eV,
		3.760*eV, 3.877*eV, 4.002*eV, 4.136*eV };
	
	G4double RefractiveIndex_Water[nEntries_Water] =
	{ 1.3435  , 1.344   , 1.3445  , 1.345   ,
		1.3455  , 1.346   , 1.3465  , 1.347   ,
		1.3475  , 1.348   , 1.3485  , 1.3492  ,
		1.35    , 1.3505  , 1.351   , 1.3518  ,
		1.3522  , 1.3530  , 1.3535  , 1.354   ,
		1.3545  , 1.355   , 1.3555  , 1.356   ,
		1.3568  , 1.3572  , 1.358   , 1.3585  ,
		1.359   , 1.3595  , 1.36    , 1.3608   };
	
	G4double Absorption_Water[nEntries_Water] =
	{  3.448*m ,  4.082*m,  6.329*m,  9.174*m,
		12.346*m , 13.889*m, 15.152*m, 17.241*m,
		18.868*m , 20.000*m, 26.316*m, 35.714*m,
		45.455*m , 47.619*m, 52.632*m, 52.632*m,
		55.556*m , 52.632*m, 52.632*m, 47.619*m,
		45.455*m , 41.667*m, 37.037*m, 33.333*m,
		30.000*m , 28.500*m, 27.000*m, 24.500*m,
		22.000*m , 19.500*m, 17.500*m, 14.500*m };
	
	G4double ScintilFast_Water[nEntries_Water] =
	{ 1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00   ,
		1.00    ,  1.00   ,  1.00   ,  1.00    };
	
	G4double ScintilSlow_Water[nEntries_Water] =
	{ 0.01    ,  1.00   ,  2.00   ,  3.00   ,
		4.00    ,  5.00   ,  6.00   ,  7.00   ,
		8.00    ,  9.00   ,  8.00   ,  7.00   ,
		6.00    ,  4.00   ,  3.00   ,  2.00   ,
		1.00    ,  0.01   ,  1.00   ,  2.00   ,
		3.00    ,  4.00   ,  5.00   ,  6.00   ,
		7.00    ,  8.00   ,  9.00   ,  8.00   ,
		7.00    ,  6.00   ,  5.00   ,  4.00    };
	
	
	G4MaterialPropertiesTable* Water_MPT = new G4MaterialPropertiesTable();
	Water_MPT->AddProperty("RINDEX",        PhotonEnergy_Water, RefractiveIndex_Water, nEntries_Water);
	Water_MPT->AddProperty("ABSLENGTH",     PhotonEnergy_Water, Absorption_Water,      nEntries_Water);
	Water_MPT->AddProperty("FASTCOMPONENT", PhotonEnergy_Water, ScintilFast_Water,     nEntries_Water);
	Water_MPT->AddProperty("SLOWCOMPONENT", PhotonEnergy_Water, ScintilSlow_Water,     nEntries_Water);
	
	Water_MPT->AddConstProperty("SCINTILLATIONYIELD", 10./MeV);
	Water_MPT->AddConstProperty("RESOLUTIONSCALE",    1.0);
	Water_MPT->AddConstProperty("FASTTIMECONSTANT",   1.*ns);
	Water_MPT->AddConstProperty("SLOWTIMECONSTANT",  10.*ns);
	Water_MPT->AddConstProperty("YIELDRATIO",         0.8);
	
	MMats["Water"]->SetMaterialPropertiesTable(Water_MPT);
	MMats["H2O"]->SetMaterialPropertiesTable(Water_MPT);
	
	
	// Cerenkov Gas properties
	const G4int nEntries_CCGas = 2;
	G4double PhotonEnergy_CCGas[nEntries_CCGas]    = { 2.034*eV, 4.136*eV };
	G4double RefractiveIndex_CCGas[nEntries_CCGas] = { 1.001331  , 1.00143   };
	G4double Absorption_CCGas[nEntries_CCGas]      = { 10*m ,  3*m };
	
	G4MaterialPropertiesTable* CCGas_MPT = new G4MaterialPropertiesTable();
	CCGas_MPT->AddProperty("RINDEX",        PhotonEnergy_CCGas, RefractiveIndex_CCGas, nEntries_CCGas);
	CCGas_MPT->AddProperty("ABSLENGTH",     PhotonEnergy_CCGas, Absorption_CCGas,      nEntries_CCGas);
	CCGas_MPT->AddConstProperty("SCINTILLATIONYIELD", 10./MeV);
	CCGas_MPT->AddConstProperty("RESOLUTIONSCALE",    1.0);
	CCGas_MPT->AddConstProperty("YIELDRATIO",         0.8);
	MMats["CCGas"]->SetMaterialPropertiesTable(CCGas_MPT);
	
	
	// Aerogel properties
	const G4int nEntries_Aerogel = 2;
	G4double PhotonEnergy_Aerogel[nEntries_Aerogel]    = { 2.034*eV, 4.136*eV };
	G4double RefractiveIndex_Aerogel[nEntries_Aerogel] = { 1.03  , 1.03   };
	G4double Absorption_Aerogel[nEntries_Aerogel]      = { 10*m ,  3*cm };
	
	G4MaterialPropertiesTable* Aerogel_MPT = new G4MaterialPropertiesTable();
	Aerogel_MPT->AddProperty("RINDEX",        PhotonEnergy_Aerogel, RefractiveIndex_Aerogel, nEntries_Aerogel);
	Aerogel_MPT->AddProperty("ABSLENGTH",     PhotonEnergy_Aerogel, Absorption_Aerogel,      nEntries_Aerogel);
	Aerogel_MPT->AddConstProperty("SCINTILLATIONYIELD", 10./MeV);
	Aerogel_MPT->AddConstProperty("RESOLUTIONSCALE",    1.0);
	Aerogel_MPT->AddConstProperty("YIELDRATIO",         0.8);	
	MMats["Aerogel"]->SetMaterialPropertiesTable(Aerogel_MPT);
	
	
	// RICH
	const G4int nEntries_Rich = 14;
	
  G4double PhotonEnergy_Rich[nEntries_Rich] = 
  { 2.21*eV, 2.30*eV, 2.38*eV, 2.48*eV,
		2.58*eV, 2.70*eV, 2.82*eV, 2.95*eV,
		3.10*eV, 3.26*eV, 3.44*eV, 3.65*eV,
		3.88*eV, 4.13*eV};
	
	
  G4double C5F12_Rind[nEntries_Rich] =
  { 1.23862, 1.23884, 1.23906, 1.23933,
		1.23962, 1.23998, 1.24035, 1.24078,
		1.24130, 1.24189, 1.24259, 1.24346,
		1.24448, 1.24567};
  
  G4double C5F12_Abs[nEntries_Rich] =
  { 2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm};  
	
	
  G4MaterialPropertiesTable* C5F12_MPT = new G4MaterialPropertiesTable();
  C5F12_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, C5F12_Rind, nEntries_Rich);
  C5F12_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, C5F12_Abs,  nEntries_Rich);
  MMats["C5F12"]->SetMaterialPropertiesTable(C5F12_MPT);
	
	
	
	
	G4double RichAerogel_Rind[nEntries_Rich] =
  { 1.06, 1.06, 1.06, 1.06, 
		1.06, 1.06, 1.06, 1.06, 
		1.06, 1.06, 1.06, 1.06,
		1.06, 1.06};  
  
	G4double RichAerogel_Abs[nEntries_Rich] =
  { 20.309*cm, 17.669*cm, 15.610*cm, 13.515*cm, 
    11.720*cm,  9.756*cm,  8.315*cm,  6.941*cm, 
     5.699*cm,  4.649*cm,  3.727*cm,  2.928*cm, 
     2.253*cm,  1.697*cm};
	
	
  G4MaterialPropertiesTable* RichAerogel_MPT = new G4MaterialPropertiesTable();
  RichAerogel_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, RichAerogel_Rind, nEntries_Rich);
  RichAerogel_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, RichAerogel_Abs,  nEntries_Rich);
  MMats["RichAerogel"]->SetMaterialPropertiesTable(RichAerogel_MPT);
	
	
  G4double C6F14_Rind[nEntries_Rich] =
  { 1.21501, 1.21656, 1.21794, 1.21966,
		1.22138, 1.22344, 1.22550, 1.22774,
		1.23032, 1.23307, 1.23617, 1.23978,
		1.24374, 1.24804};
	
  G4double C6F14_Abs[nEntries_Rich] =
  { 2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm, 2000.*mm, 2000.*mm, 
		2000.*mm, 2000.*mm};
	
  G4MaterialPropertiesTable* C6F14_MPT = new G4MaterialPropertiesTable();
  C6F14_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, C6F14_Rind, nEntries_Rich);
  C6F14_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, C6F14_Abs,  nEntries_Rich);
  MMats["C6F14"]->SetMaterialPropertiesTable(C6F14_MPT);
	
	
  G4double Quartz_Rind[nEntries_Rich] =
  { 1.505, 1.509, 1.511, 1.515, 
		1.520, 1.525, 1.528, 1.527, 
		1.522, 1.512, 1.505, 1.492, 
		1.471, 1.503};
	
  G4double Quartz_Abs[nEntries_Rich] =
  { 550.7*mm, 530.7*mm, 590.1*mm, 490.7*mm,
		470.7*mm, 520.3*mm, 500.0*mm, 470.7*mm,
		450.5*mm, 270.5*mm, 190.1*mm,  60.9*mm,
     10.6*mm,   4.0*mm};
	
  G4MaterialPropertiesTable* Quartz_MPT = new G4MaterialPropertiesTable();
  Quartz_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, Quartz_Rind, nEntries_Rich);
  Quartz_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, Quartz_Abs,  nEntries_Rich);
  MMats["Quartz"]->SetMaterialPropertiesTable(Quartz_MPT);
	
  G4double Methane_Rind[nEntries_Rich] =
  { 1., 1., 1., 1., 1., 1., 1., 
		1., 1., 1., 1., 1., 1., 1.};
	
  G4double Methane_Abs[nEntries_Rich] =
  {4000.*cm,4000.*cm,4000.*cm, 4000.*cm,
		4000.*cm,4000.*cm,4000.*cm, 4000.*cm,
		4000.*cm,4000.*cm};
	
	
  G4MaterialPropertiesTable* Methane_MPT = new G4MaterialPropertiesTable();
  Methane_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, Methane_Rind, nEntries_Rich);
  Methane_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, Methane_Abs,  nEntries_Rich);
  MMats["Methane"]->SetMaterialPropertiesTable(Methane_MPT);


  G4double Alumi_Rind[nEntries_Rich] =
  { 1., 1., 1., 1., 1., 1., 1., 
		1., 1., 1., 1., 1., 1., 1.};
	
  G4double Alumi_Abs[nEntries_Rich] =
  {0., 0., 0., 0., 0., 0., 0., 
		0., 0., 0., 0., 0., 0., 0.};
	
  G4double Alumi_Effi[nEntries_Rich] =
  {0., 1., 1., 1., 1., 1., 1., 
		1., 1., 1., 1., 1., 1., 0.};
	
  G4double Alumi_Refl[nEntries_Rich] =
  {0., 0., 0., 0., 0., 0., 0., 
		0., 0., 0., 0., 0., 0., 0.};
	
  G4MaterialPropertiesTable* Alumi_MPT = new G4MaterialPropertiesTable();
  Alumi_MPT->AddProperty("RINDEX",       PhotonEnergy_Rich, Alumi_Rind, nEntries_Rich);
  Alumi_MPT->AddProperty("ABSLENGTH",    PhotonEnergy_Rich, Alumi_Abs,  nEntries_Rich);
  Alumi_MPT->AddProperty("EFFICIENCY",   PhotonEnergy_Rich, Alumi_Effi, nEntries_Rich);
  Alumi_MPT->AddProperty("REFLECTIVITY", PhotonEnergy_Rich, Alumi_Refl, nEntries_Rich);
  MMats["Alumi"]->SetMaterialPropertiesTable(Alumi_MPT);
	
  G4double Glass_Rind[nEntries_Rich] =
  {1.49, 1.49, 1.49, 1.49,
		1.49, 1.49, 1.49, 1.49,
		1.49, 1.49, 1.49, 1.49,
		1.49, 1.49};
	
  G4double Glass_Abs[nEntries_Rich] =
  {4200.0*mm, 4200.0*mm, 4200.0*mm, 4200.0*mm, 
		4200.0*mm, 4200.0*mm, 4200.0*mm, 4200.0*mm, 
		4200.0*mm, 4200.0*mm, 4200.0*mm, 4200.0*mm, 
		4200.0*mm, 4200.0*mm};
	
	
  G4MaterialPropertiesTable* Glass_MPT = new G4MaterialPropertiesTable();
  Glass_MPT->AddProperty("RINDEX",     PhotonEnergy_Rich, Glass_Rind, nEntries_Rich);
  Glass_MPT->AddProperty("ABSLENGTH",  PhotonEnergy_Rich, Glass_Abs,  nEntries_Rich);
  MMats["Glass"]->SetMaterialPropertiesTable(Glass_MPT);
  //MMats["Vacuum"]->SetMaterialPropertiesTable(Glass_MPT);
	
	
	return MMats;
}



